'From Squeak 2.4c of May 10, 1999 on 10 June 1999 at 11:47:11 am'!
"Change Set:		NewWorld3
Date:			10 June 1999
Author:			Dan Ingalls

Adds a few changes to allow a new project to be opened with a given PasteUpMorph as its world.

Also fixes a bug reported by Ted that MorphicProjects were not showing up in the navigation lists.
"!


!HandMorph methodsFor: 'world menu commands' stamp: 'di 6/9/1999 15:45'!
openMorphicProject

	(ProjectViewMorph newMorphicProjectOn: nil) openInWorld.
! !


!PasteUpMorph methodsFor: 'project' stamp: 'di 6/9/1999 15:43'!
beWorldForProject: aProject

	project _ aProject.
	self privateOwner: nil.
	self hands: Array new.
	self addHand: HandMorph new.
	self damageRecorder: DamageRecorder new.
	self stepList: OrderedCollection new.
	self lastStepTime: 0.
	self setProperty: #automaticPhraseExpansion toValue: true.
	self startSteppingSubmorphsOf: self! !


!Project methodsFor: 'accessing' stamp: 'di 6/10/1999 11:44'!
name
	changeSet == nil ifTrue: [^ nil].
	^ changeSet name! !

!Project methodsFor: 'release' stamp: 'di 6/10/1999 11:33'!
okToChange
	| ok hasSubProjects |
	hasSubProjects _ world isMorph
		ifTrue: [(world submorphs select:
						[:m | (m isKindOf: SystemWindow)
								and: [m model isKindOf: Project]]) size > 0]
		ifFalse: [(world controllerWhoseModelSatisfies:
						[:m | m isKindOf: Project]) notNil].
	hasSubProjects ifTrue:
		[PopUpMenu notify: 
'The project ', self name printString, '
contains sub-projects.  You must remove these
explicitly before removing their parent.'.
		^ false].
	ok _ (world isMorph not and: [world scheduledControllers size <= 1]) or:
			[self confirm:
'Really delete the project
', self name printString, '
and all its windows?'].
	ok ifFalse: [^ false].

	"about to delete this project; clear previous links to it from other Projects:"
	Project allProjects do: [:p | p deletingProject: self].
	ProjectViewMorph allInstancesDo: [:p | p deletingProject: self].

	world isMorph  "special release for wonderlands"
		ifTrue: [world submorphs do:
						[:m | (m isKindOf: WonderlandCameraMorph)
								and: [m getWonderland release]]].

	^ true
! !


!MorphicProject methodsFor: 'initialization' stamp: 'di 6/9/1999 15:16'!
initFromParent:  aProject

	"Written so that Morphic can still be removed."
	Smalltalk verifyMorphicAvailability ifFalse: [^ nil].
	world _ PasteUpMorph newWorldForProject: self.
	changeSet _ ChangeSet new initialize.
	transcript _ TranscriptStream new.
	displayDepth _ Display depth.
	parentProject _ aProject! !

!MorphicProject methodsFor: 'initialization' stamp: 'di 6/9/1999 15:47'!
installPasteUpAsWorld: pasteUpMorph
	"(ProjectViewMorph newMorphicProjectOn: aPasteUpMorph) openInWorld."

	world _ pasteUpMorph beWorldForProject: self! !


!Project class methodsFor: 'utilities' stamp: 'di 6/10/1999 11:30'!
allNames
	^ (self allProjects collect: [:p | p name]) asSortedCollection: [:n1 :n2 | n1 asLowercase < n2 asLowercase]! !

!Project class methodsFor: 'utilities' stamp: 'di 6/10/1999 11:30'!
allNamesAndProjects
	^ (self allProjects asSortedCollection: [:p1 :p2 | p1 name asLowercase < p2 name asLowercase]) collect:
		[:aProject | Array with: aProject name with: aProject]! !

!Project class methodsFor: 'utilities' stamp: 'di 6/10/1999 11:44'!
allProjects

	^ self allSubInstances select: [:p | p name notNil]! !

!Project class methodsFor: 'utilities' stamp: 'di 6/10/1999 11:31'!
named: projName
	"Answer the project with the given name, or the current project if there is no project of that given name."
	"(Project named: 'zork') enter"

	^ self allProjects
		detect: [:proj | proj name = projName]
		ifNone: [self current]
! !


!MorphicProject class reorganize!
('instance creation' newMorphic newMorphicConstruction)
!


!MorphicProject class methodsFor: 'instance creation' stamp: 'di 6/9/1999 15:13'!
newMorphic   "ProjectView open: MorphicProject newMorphic"

	^ self basicNew initFromParent: CurrentProject! !

!MorphicProject class methodsFor: 'instance creation' stamp: 'di 6/9/1999 15:19'!
newMorphicConstruction
	"ProjectView open: MorphicProject newMorphicConstruction"

	| aProject |
	aProject _ self newMorphic.
	aProject world configureForConstruction.
	^ aProject! !


!ProjectViewMorph class methodsFor: 'project window creation' stamp: 'di 6/9/1999 15:46'!
newMorphicProjectOn: aPasteUpOrNil
	"Return an instance of me on a new Morphic project (in a SystemWindow)."

	| proj window |
	proj _ MorphicProject newMorphic.
	aPasteUpOrNil ifNotNil: [proj installPasteUpAsWorld: aPasteUpOrNil].
	window _ (SystemWindow labelled: proj name) model: proj.
	window
		addMorph: (self on: proj)
		frame: (0@0 corner: 1.0@1.0).
	^ window
! !


Project removeSelector: #initFromParent:!
Project class removeSelector: #newMorphicConstruction!
Project class removeSelector: #newMorphic!
