'From Squeak 2.4c of May 10, 1999 on 7 June 1999 at 10:21:35 pm'!
"Change Set:		gcPolicy-jhm
Date:			7 June 1999
Author:			John Maloney

This change adds a mechanism for tweaking the default GC rate and
tenuring threshold. This can be used to reduce tenuring in the (fairly
rare) cases when the default VM tenuring threshold causes over-eager
tenuring. This change set adds the mechanism, but does not actually change
the defaults. To do so, you would edit SystemDictionary>setGCParameters
(for example, 7000 allocations between GCs, 6000 survivor tenure threshold)."!

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'jm 6/7/1999 21:33'!
setGCParameters
	"Adjust the VM's default GC parameters to avoid premature tenuring."

	Smalltalk vmParameterAt: 5 put: 4000.  "do an incremental GC after this many allocations"
	Smalltalk vmParameterAt: 6 put: 2000.  "tenure when more than this many objects survive the GC"
! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'jm 6/7/1999 21:33'!
snapshot: save andQuit: quit
	"Mark the changes file and close all files. If save is true, save the current state of this Smalltalk in the image file. If quit is true, then exit to the outer shell. The latter part of this method runs when resuming a previously saved image. The resume logic checks for a document file to process when starting up."

	| resuming msg sourceLink |
	save & (SourceFiles at: 2) notNil ifTrue:
		[msg _  (quit
			ifTrue: ['----QUIT----']
			ifFalse: ['----SNAPSHOT----'])
			, Date dateAndTimeNow printString.
		sourceLink _ ' priorSource: ' , LastQuitLogPosition printString.
		self assureStartupStampLogged.
		LastQuitLogPosition _ (SourceFiles at: 2) setToEnd; position.
		self logChange: msg , sourceLink.
		Transcript cr; show: msg].

	self processShutDownList.
	Cursor write show.
	save
		ifTrue: [resuming _ self snapshotPrimitive]  "<-- PC frozen here on image file"
		ifFalse: [resuming _ false].
	quit & resuming not ifTrue: [self quitPrimitive].
	Cursor normal show.
	self setGCParameters.
	self processStartUpList.
	resuming ifTrue: [
		self clearExternalObjects.
		self readDocumentFile].
! !

"Postscript:
Set the GC parameters to decrease the probability of tenuring when running two SubGames."
Smalltalk setGCParameters.
!
