'From Squeak 2.1 of June 30, 1998 on 15 July 1998 at 11:49:22 pm'!
"Change Set:		DirectoryEntry
Date:			15 July 1998
Author:			Lex Spoon

Make FileDirectory>>entries return DirectoryEntries instead of Array's.  Updates ServerDirectory as well.  Now you can say {entry fileSize} instead of {entry at: 5}"!

ArrayedCollection subclass: #DirectoryEntry
	instanceVariableNames: 'name creationTime modificationTime dirFlag fileSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Files'!

!DirectoryEntry commentStamp: 'ls 7/15/1998 23:49' prior: 0!
an entry in a directory; a reference to either a file or a directory.!

!DirectoryEntry reorganize!
('access' creationTime fileSize isDirectory modificationTime name)
('access-compatibility' at: size)
('private-initialization' privateName:creationTime:modificationTime:isDirectory:fileSize:)
!


!DirectoryEntry methodsFor: 'access' stamp: 'ls 7/15/1998 21:37'!
creationTime
	"time the entry was created.  (what's its type?)"
	^creationTime! !

!DirectoryEntry methodsFor: 'access' stamp: 'ls 7/15/1998 21:38'!
fileSize
	"size of the entry, if it's a file"
	^fileSize! !

!DirectoryEntry methodsFor: 'access' stamp: 'ls 7/15/1998 21:38'!
isDirectory
	"whether this entry represents a directory"
	^dirFlag! !

!DirectoryEntry methodsFor: 'access' stamp: 'ls 7/15/1998 21:37'!
modificationTime
	"time the entry was last modified"
	^modificationTime! !

!DirectoryEntry methodsFor: 'access' stamp: 'ls 7/15/1998 21:37'!
name
	"name of the entry"
	^name! !

!DirectoryEntry methodsFor: 'access-compatibility' stamp: 'ls 7/15/1998 22:29'!
at: index
	"compatibility interface"
	"self halt: 'old-style access to DirectoryEntry'"
	index = 1 ifTrue: [ ^self name ].
	index = 2 ifTrue: [ ^self creationTime ].
	index = 3 ifTrue: [ ^self modificationTime ].
	index = 4 ifTrue:[ ^self isDirectory ].
	index = 5 ifTrue:[ ^self fileSize ].
	self error: 'invalid index specified'.! !

!DirectoryEntry methodsFor: 'access-compatibility' stamp: 'ls 7/15/1998 22:16'!
size
	^5! !

!DirectoryEntry methodsFor: 'private-initialization' stamp: 'ls 7/15/1998 21:42'!
privateName: name0  creationTime: creationTime0  modificationTime: modificationTime0  isDirectory: isDirectory0  fileSize: fileSize0
	name _ name0.
	creationTime _ creationTime0.
	modificationTime _ modificationTime0.
	dirFlag _ isDirectory0.
	fileSize _ fileSize0.! !


!DirectoryEntry class reorganize!
('instance creation' fromArray: name:creationTime:modificationTime:isDirectory:fileSize:)
!


!DirectoryEntry class methodsFor: 'instance creation' stamp: 'ls 7/15/1998 21:42'!
fromArray: array
	^self name: (array at: 1) creationTime: (array at: 2) modificationTime: (array at: 3) isDirectory: (array at: 4) fileSize: (array at: 5) ! !

!DirectoryEntry class methodsFor: 'instance creation' stamp: 'ls 7/15/1998 21:41'!
name: name0  creationTime: creationTime  modificationTime: modificationTime   isDirectory: isDirectory  fileSize: fileSize
	^self new privateName: name0  creationTime: creationTime  modificationTime: modificationTime  isDirectory: isDirectory  fileSize: fileSize! !


!FileDirectory methodsFor: 'private' stamp: 'ls 7/15/1998 21:40'!
directoryContentsFor: fullPath
	"Return a collection of directory entries for the files and directories in the directory with the given path. See primLookupEntryIn:index: for further details."
	"FileDirectory default directoryContentsFor: ''"

	| entries index done entryArray |
	entries _ OrderedCollection new: 200.
	index _ 1.
	done _ false.
	[done] whileFalse: [
		entryArray _ self primLookupEntryIn: fullPath index: index.
		entryArray == nil
			ifTrue: [done _ true]
			ifFalse: [entries addLast: (DirectoryEntry fromArray: entryArray)].
		index _ index + 1].

	^ entries asArray
! !


!ServerDirectory methodsFor: 'file directory' stamp: 'ls 7/15/1998 22:11'!
entries
	"Return a collection of directory entries for the files and directories in this directory. Each entry is a five-element array: (<name><creationTime><modificationTime><dirFlag><fileSize>)."

	| ftpEntries thisYear tokens dateInSeconds longy |
	"We start with ftp directory entries of the form...
d---------   1 owner    group               0 Apr 27 22:01 blasttest
----------   1 owner    group           93812 Jul 21  1997 COMMAND.COM
    1        2   3           4                 5    6  7    8       9   -- token index"
	ftpEntries _ self getDirectory contentsOfEntireFile findTokens: FTPSocket crLf.

	thisYear _ Date today year.
	^ ftpEntries collect:
		[:ftpEntry | tokens _ ftpEntry findTokens: ' '.
		tokens size >= 9
		ifTrue:
		[tokens size > 9 ifTrue:
			[longy _ tokens at: 9.
			10 to: tokens size do: [:i | longy _ longy , ' ' , (tokens at: i)].
			tokens at: 9 put: longy].
		dateInSeconds _ ((tokens at: 8) includes: $:)
			ifTrue: ["Date has no year -- assume this year"
					(Date newDay: (tokens at: 7) asNumber
								month: (tokens at: 6)
								year: thisYear) asSeconds +
					(Time readFrom: (ReadStream on: (tokens at: 8))) asSeconds]
			ifFalse: [(Date newDay: (tokens at: 7) asNumber
								month: (tokens at: 6)
								year: (tokens at: 8) asNumber) asSeconds].
		DirectoryEntry name: (tokens at: 9)  "file name"
			creationTime: dateInSeconds "creation date"
			modificationTime: dateInSeconds "modification time"
			isDirectory: tokens first first = $d "is-a-directory flag"
			fileSize: tokens fifth asNumber "file size"]
		ifFalse:
		[nil]]
		thenSelect: [:entry | entry notNil]! !


