'From Squeak2.9alpha of 13 June 2000 [latest update: #2426] on 25 June 2000 at 4:22:19 pm'!
"Change Set:		171CelesteFastDelete-ls
Date:			20 June 2000
Author:			Lex Spoon

Makes removeMessage and removeAll much faster: instead of regenerating the entire message list, it simply removes one item.  Also, updates move* and delete* to share code with remove*, and modifies file* to not update the contents list at all."
!


!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:01'!
deleteAll
	"Move all visible messages in the current category to '.trash.'."

	| |

	self requiredCategory: '.trash.'.

	mailDB fileAll: currentMessages inCategory: '.trash.'.

	self removeAll.! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 10:44'!
deleteMessage
	"Move the current message to the '.trash.' category and select the next message. Deleted messages can later purged by the 'empty trash' menu item"

	currentMsgID isNil ifTrue: [^ self].
	self requiredCategory: '.trash.'.

	mailDB file: currentMsgID inCategory: '.trash.'.

	self removeMessage.
! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:11'!
fileAgain
	"File the current message in the same category as last time."

	| newCatName |
	(lastCategory isEmpty not)
		ifTrue: [newCatName _ lastCategory]
		ifFalse: [newCatName _ self getCategoryNameIfNone: [^self]].
	mailDB file: currentMsgID inCategory: newCatName.
! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:07'!
fileMessage
	"File the current message in another category."

	| newCatName |
	newCatName _ self getCategoryNameIfNone: [^self].
	mailDB file: currentMsgID inCategory: newCatName.
! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:08'!
moveAgain
	"Move the current message to the same category as last time."

	| newCatName |
	currentMsgID ifNil: [ ^self ].
	(lastCategory isEmpty not)
		ifTrue: [newCatName _ lastCategory]
		ifFalse: [newCatName _ self getCategoryNameIfNone: [^self]].
	newCatName = currentCategory ifTrue: [ ^self ].

	mailDB remove: currentMsgID fromCategory: currentCategory.

	self removeMessage.! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:09'!
moveAll
	"Move all visible messages in the current category to another category."

	| newCatName |
	newCatName _ self getCategoryNameIfNone: [^self].
	newCatName = currentCategory ifTrue:[ ^self ].
	mailDB fileAll: currentMessages inCategory: newCatName.

	self removeAll.! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:08'!
moveMessage
	"Move the current message to another category; this consists of filing it in the new category, and then removing it from the current category"

	| newCatName |
	newCatName _ self getCategoryNameIfNone: [^self].
	newCatName = currentCategory ifTrue: [ ^self ].
	mailDB file: currentMsgID inCategory: newCatName.

	self removeMessage.
! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:00'!
removeAll
	"Remove all messages from the current category."

	| |
	mailDB removeAll: currentMessages fromCategory: currentCategory.
	currentMsgID _ nil.
	currentMessages _ #().
	currentTOC _ #().


	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 10:59'!
removeMessage
	| currentMessageIndex |
	"Remove the current message from the current category."

	currentMsgID ifNil: [ ^self ].

	mailDB remove: currentMsgID fromCategory: currentCategory.

	"remove the message from the listing"
	currentMessageIndex _ currentMessages indexOf: currentMsgID.
	currentMessages _ currentMessages copyWithout: currentMsgID.
	currentTOC := currentTOC copyWithoutIndex: currentMessageIndex.

	"update the message index and message ID"
	currentMessages isEmpty ifTrue: [
		currentMsgID _ nil ]
	ifFalse: [
		currentMsgID _ currentMessages at: (currentMessageIndex min: currentMessages size) ].


	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.
! !

!Celeste methodsFor: 'table of contents pane' stamp: 'ls 6/20/2000 11:33'!
updateTOC
	"Update the table of contents after a moving, removing, or deleting a message. Select a message near the removed message in the table of contents if possible."

	| savedMsgID |
	"remember the currently selected message"
	savedMsgID _ currentMsgID.

	"update the TOC listing"
	currentMsgID _ nil.
	self setCategory: currentCategory.  "update currentMessages, currentTOC"

	"try to select the previously selected message; if impossible, select the first message"
	currentMessages isEmptyOrNil ifFalse: [
		(currentMessages includes: savedMsgID) 
			ifTrue: [ currentMsgID _ savedMsgID ]
			ifFalse: [ currentMsgID _ currentMessages first ] ].

	"NB: self changed: #tocEntryList  is already done above by setCategory: and can be slow"
	self changed: #tocEntry.
	self changed: #messageText.
! !


!SequenceableCollection methodsFor: 'copying' stamp: 'ls 6/20/2000 10:59'!
copyWithoutIndex: index
	| copy |
	"return a copy containing all elements except the index-th"
	copy := self species new: self size - 1.
	copy replaceFrom: 1 to: index-1 with: self startingAt: 1.
	copy replaceFrom: index to: copy size with: self startingAt: index+1.
	^copy! !

