'From Squeak2.9alpha of 5 August 2000 [latest update: #2580] on 8 September 2000 at 9:44:37 pm'!
"Change Set:		eToyAdditions-sw
Date:			8 September 2000
Author:			Scott Wallace

Additions to the etoy scripting vocabulary:

For playfields/holders:
  *  append: -- adds the object provided to the playfield, after all other morphs already on the playfield.
  *  removeAll -- empties out the container.
  *  shuffleContents -- rearranges subparts in random order.
  *  numberAtCursor -- delivers the number represented by the morph at the current cursor position.  A direct numeric counterpart for valueAtCursor.

For every morph
  *  copy -- returns a sibling object with appearance and behavior the same as the original.
  *  elementNumber -- tell which element-number an object is from the perspective of its container.
  *  numericValue -- (primarily for TextMorphs and Sliders, which have ways of displaying that value, but generically implemented for all morphs.
  *  obtrudes -- a true/false value for use in TEST panes; true if any part of the object sticks out beyond the bounds of its container

For text morphs
  *  characters -- returns the string of characters held.
  *  firstCharacter -- returns the initial character.

There is also a complete reworking of the way that the contents of Viewers are defined, the point being to centralize all the changes you need to make to add a new item or category or to change any one is concentrated in one method.  See implementors of #additionsToViewerCategories.

Balloon help is now provided both for viewer and scriptor phrase tiles.

This update also contains numerous other fixups and feature additions and changes in etoyland, primarily in preparation for forthcoming classroom use 10/00.
"!

Object subclass: #StandardScriptingSystem
	instanceVariableNames: ''
	classVariableNames: 'CategoryElementDictionary CustomPartsBin FormDictionary HelpStrings ReservedScriptNames  StandardPartsBin StandardScriptInfo StandardSlotInfo SystemSlotDictionary TypeColorDictionary '
	poolDictionaries: ''
	category: 'Morphic-Scripting'!


!Morph methodsFor: 'copying' stamp: 'sw 9/1/2000 10:18'!
usableSiblingInstance
	"Return another similar morph whose Player is of the same class as mine"

	|  aName usedNames newPlayer newMorph |
	self flag: #noteToTed.  "I think this may have deviated majorly from your mainstream veryDeepCopy work, so there will probably be some problems entailed"

	newMorph _ self veryDeepCopyWithoutPlayer.
	newPlayer _ self player class new costume: newMorph.
	self isFlexMorph ifTrue: [newMorph renderedMorph player: newPlayer]. "???"
	newMorph actorState: (self player actorState shallowCopy initializeFor: newPlayer).

	(aName _ self knownName) == nil ifTrue:
		[self player ~~ nil ifTrue: [aName _ newMorph innocuousName]].
			"Force a difference here"
	aName ~~ nil ifTrue:
		[usedNames _ (self world ifNil: [OrderedCollection new] ifNotNil: [self world allKnownNames]) copyWith: aName.
		newMorph setNameTo: (Utilities keyLike: aName satisfying: [:f | (usedNames includes: f) not])].
	"newMorph justDuplicatedFrom: self.  NOT done for sibling inst"
	newMorph privateOwner: nil.
	(newMorph renderedMorph eventHandler ~~ nil) ifTrue:
		[newPlayer assureEventHandlerRepresentsStatus].
	self presenter flushPlayerListCache.

	^ newMorph! !

!Morph methodsFor: 'submorphs-accessing' stamp: 'sw 9/1/2000 07:17'!
shuffleSubmorphs
	"Randomly shuffle the order of my submorphs.  Don't call this method lightly!!"

	| bg |
	(submorphs size > 0 and: [submorphs last mustBeBackmost]) ifTrue:
		[bg _ submorphs last.
		bg privateDelete].
	submorphs _ submorphs shuffled.
	bg ifNotNil: [self addMorphBack: bg].
	self layoutChanged! !

!Morph methodsFor: 'submorphs-add/remove' stamp: 'sw 9/7/2000 08:29'!
addMorph: aMorph asElementNumber: aNumber
	"Add the given morph so that it becomes the aNumber'th element of my submorph list.  If aMorph is already one of my submorphs, reposition it"

	(submorphs includes: aMorph) ifTrue:
		[aMorph privateDelete].
	(aNumber <= submorphs size)
		ifTrue:
			[self addMorph: aMorph inFrontOf: (submorphs at: aNumber)]
		ifFalse:
			[self addMorphBack: aMorph]
! !

!Morph methodsFor: 'submorphs-add/remove' stamp: 'sw 9/1/2000 10:16'!
submorphIndexOf: aMorph
	"Assuming aMorph to be one of my submorphs, answer where it occurs in my submorph list"

	^ submorphs indexOf: aMorph ifAbsent: [nil]! !

!Morph methodsFor: 'geometry testing' stamp: 'sw 9/7/2000 12:19'!
obtrudesBeyondContainer
	"Answer whether the receiver obtrudes beyond the bounds of its container"

	| top |
	top _ self topRendererOrSelf.
	(top owner == nil or: [top owner isHandMorph]) ifTrue: [^ false].
	^ (top owner bounds containsRect: top bounds) not
	! !

!Morph methodsFor: 'geometry eToy' stamp: 'sw 9/8/2000 16:35'!
getIndexInOwner
	"Answer which position the receiver holds in its owner's hierarchy"

	"NB: There is some concern about submorphs that aren't really to be counted, such as a background morph of a playfield."

	| container topRenderer |
	container _ (topRenderer _ self topRendererOrSelf) owner.
	^ container submorphIndexOf: topRenderer.! !

!Morph methodsFor: 'geometry eToy' stamp: 'sw 8/31/2000 11:18'!
setIndexInOwner: anInteger
	"Answer which position the receiver holds in its owner's hierarchy"

	"There is some concern about submorphs that aren't really to be counted, such as a background morph of a playfield."
	| container topRenderer indexToUse |
	container _ (topRenderer _ self topRendererOrSelf) owner.
	indexToUse _ (anInteger min: container submorphCount) max: 1.
	container addMorph: topRenderer asElementNumber: indexToUse! !

!Morph methodsFor: 'scripting' stamp: 'sw 9/8/2000 16:32'!
categoriesForViewer
	"Answer a list of symbols representing the categories to offer in the viewer, in order"

	| aClass aList predetermined genericItems |
	aClass _ self renderedMorph class.
	aList _ OrderedCollection new.
	[aClass == Morph] whileFalse:
		[(aClass class includesSelector: #additionsToViewerCategories) ifTrue:
			[aList addAllFirst: (aClass additionsToViewerCategories collect:
				[:categorySpec | categorySpec first])].
		aClass _ aClass superclass]. 

	genericItems _ Morph additionsToViewerCategories collect:
			[:categorySpec | categorySpec first].
	aList removeAllFoundIn: genericItems.

	aList addAllFirst: (Morph additionsToViewerCategories collect:
			[:categorySpec | categorySpec first]) asSet asOrderedCollection.
	predetermined _ #(basic #'color & border' geometry motion #'pen use' tests miscellaneous) select:
		[:sym | aList includes: sym].  "bulletproof agains change in those names elsewhere"
	aList removeAllFoundIn: predetermined.
	^ predetermined, aList
! !

!Morph methodsFor: 'scripting' stamp: 'sw 9/8/2000 16:33'!
categoryElementsFor: aCategorySymbol
	"Answer a list of tuples that characterize the elements in the given category.  Every class in the receiver's superclass chain which implements #additionsToViewerCategories is given the chance to add elements to the category"

	| aClass aList |
	aClass _ self renderedMorph class.
	aList _ OrderedCollection new.
	[aClass == Morph] whileFalse:
		[(aClass class includesSelector: #additionsToViewerCategories) ifTrue:
			[aList addAllFirst: (aClass additionsToViewerCategory: aCategorySymbol)].
		aClass _ aClass superclass]. 
	aList addAllFirst: (Morph additionsToViewerCategory: aCategorySymbol).
	^ aList
! !

!Morph methodsFor: 'scripting' stamp: 'sw 9/8/2000 11:28'!
updatingTileForArgType: typeSymbol partName: partName getSelector: getSelector putSelector: putSelector
	"Answer a readout tile representing the given part's value, given the putter, getter, and type information"

	| aColor aTile displayer |
	aColor _ Color lightGray lighter.
	aTile _ typeSymbol == #number
		ifTrue:
			[NumericReadoutTile new typeColor: aColor]
		ifFalse:	
			[typeSymbol == #sound
				ifTrue:
					[SoundReadoutTile new typeColor: aColor]
				ifFalse:
					[StringReadoutTile new typeColor: aColor]].

 	displayer _ UpdatingStringMorph new
		getSelector: getSelector;
		target: self player;
		growable: true;
		putSelector: ((putSelector == #unused) ifTrue: [nil] ifFalse: [putSelector]).

	(typeSymbol == #number)
		ifTrue:
			 [((#(cursor  "etc...") includes: partName) and: [self isKindOf: GraphMorph])
				ifTrue:
					[displayer floatPrecision: 0.1]
				ifFalse:
					[(self player slotInfo includesKey: partName)  "i.e., a user-defined numeric slot"
						ifTrue:
							[displayer floatPrecision: (self player slotInfoAt: partName) floatPrecision]]].
	typeSymbol == #string
		ifTrue:
			[displayer useStringFormat.
			displayer growable: true]
		ifFalse:
			[(typeSymbol == #sound)
				ifTrue: 	[displayer useStringFormat]
				ifFalse:	[displayer useDefaultFormat]].
	aTile addMorphBack: displayer.
	((putSelector ~~ #unused) and: [#(number sound boolean) includes: typeSymbol])  ifTrue: [aTile addArrows].
	aTile literal: (self scriptPerformer perform: getSelector).
	^ aTile! !

!Morph methodsFor: 'scripting' stamp: 'sw 9/8/2000 16:39'!
viewAfreshIn: aPasteUp showingScript: aScriptName at: aPosition
	"Obtain a smartly updated ScriptEditor for the given script name and zap it into place at aPosition"

	| anEditor |
	self player updateAllViewersAndForceToShow: #scripts.
	anEditor _ self player scriptEditorFor: aScriptName.
	aPasteUp ifNotNil: [aPasteUp addMorph: anEditor].
	anEditor position: aPosition! !

!Morph methodsFor: 'e-toy support' stamp: 'sw 9/8/2000 16:34'!
cursor
	"vacuous backstop in case it gets sent to a morph that doesn't know what to do with it"

	^ 1! !

!Morph methodsFor: 'e-toy support' stamp: 'sw 9/7/2000 09:28'!
cursor: aNumber
	"vacuous backstop in case it gets sent to a morph that doesn't know what to do with it"
! !

!Morph methodsFor: 'e-toy support' stamp: 'sw 9/1/2000 10:15'!
getNumericValue
	"Only certain kinds of morphs know how to deal with this frontally; here we provide support for a numeric property of any morph"

	^ self valueOfProperty: #numericValue ifAbsent: [0]! !

!Morph methodsFor: 'e-toy support' stamp: 'sw 9/8/2000 11:32'!
setNumericValue: aValue
	"Only certain kinds of morphs know what to do with this, the rest, for now, stash the number in a property, where it may not be visible but at least it won't be lost, and can be retrieved by the companion getter."

	(aValue ~= 0 or: [(self renderedMorph hasProperty: #numericValue) not])
		ifTrue:
			[self renderedMorph setProperty: #numericValue toValue: aValue]! !


!CategoryViewer methodsFor: 'initialization' stamp: 'sw 9/8/2000 10:58'!
initializeFor: aPlayer
	"Initialize the category pane to show the #basic category by default"

	^ self initializeFor: aPlayer categoryChoice: #basic
! !

!CategoryViewer methodsFor: 'initialization' stamp: 'sw 9/8/2000 10:58'!
initializeFor: aPlayer categoryChoice: aChoice
	"Initialize the receiver to be associated with the player and category specified"

	self orientation: #vertical;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 1;
		beSticky.
	self color: Color green muchLighter muchLighter.
	scriptedPlayer _ aPlayer.
	self addHeaderMorph.

	self categoryChoice: aChoice asSymbol
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/8/2000 10:57'!
chooseCategory
	"The mouse went down on the receiver; pop up a list of category choices"

	| aList aMenu reply aLinePosition lineList |
	aList _ scriptedPlayer categories.
	aLinePosition _ aList indexOf: #miscellaneous ifAbsent: [nil].
	lineList _ aLinePosition ifNil: [#()] ifNotNil: [Array with: aLinePosition].
	aMenu _ CustomMenu labels: aList lines: lineList selections: aList.
	reply _ aMenu startUpWithCaption: 'category'.
	reply ifNil: [^ self].
	self categoryChoice: reply asSymbol
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/8/2000 16:31'!
currentCategory
	"Answer the symbol representing the receiver's currently-selected category"

	| current actualPane |
	actualPane _ namePane renderedMorph.
	current _ (actualPane isKindOf: StringMorph)
				ifTrue:	[actualPane contents]
				ifFalse:	[actualPane firstSubmorph contents].
	^ current ifNotNil: [current asSymbol] ifNil: [#basic]! !

!CategoryViewer methodsFor: 'header pane' stamp: 'sw 9/8/2000 10:57'!
addHeaderMorph
	"Add the header at the top of the viewer, with a control for choosing the category, etc."

	| header aFont aButton wrpr |
	header _ AlignmentMorph newRow color: self color; centering: #center.
	aFont _ Preferences standardButtonFont.
	header addMorph: (aButton _ SimpleButtonMorph new label: 'O' font: aFont).
	aButton target: self;
			color:  Color tan;
			actionSelector: #delete;
			setBalloonText: 'remove this pane from the screen
don''t worry -- nothing will be lost!!.'.
	header addTransparentSpacerOfSize: 5@5.

	header addUpDownArrowsFor: self.
	(wrpr _ header submorphs last) submorphs second setBalloonText: 'previous category'.	
	wrpr submorphs first  setBalloonText: 'next category'.
	header beSticky.
	self addMorph: header.

	namePane _ RectangleMorph newSticky color: Color brown veryMuchLighter.
	namePane borderWidth: 0.
	aButton _ (StringButtonMorph contents: '-----' font: (StrikeFont familyName: #NewYork size: 12)) color: Color black.
	aButton target: self; arguments: Array new; actionSelector: #chooseCategory.
	aButton actWhen: #buttonDown.
	namePane addMorph: aButton.
	aButton position: namePane position.
	namePane align: namePane topLeft with: (bounds topLeft + (50 @ 0)).
	namePane setBalloonText: 'category (click here to choose a different one)'.

	header addMorphBack: namePane.
	(namePane isKindOf: RectangleMorph) ifTrue:
		[namePane addDropShadow.
		namePane owner color: Color gray].

	self categoryChoice: #basic! !

!CategoryViewer methodsFor: 'entries' stamp: 'sw 9/8/2000 21:35'!
infoButtonFor: aScriptOrSlotSymbol
	| aButton balloonTextSelector |
	balloonTextSelector _ nil.
	(scriptedPlayer slotInfo includesKey: aScriptOrSlotSymbol asSymbol)
		ifTrue: [balloonTextSelector _ #userSlot].

	(scriptedPlayer belongsToUniClass and:
			[scriptedPlayer class includesSelector: aScriptOrSlotSymbol])
		ifTrue: [balloonTextSelector _ #userScript].

	aButton _ SimpleButtonMorph new.
	aButton target: scriptedPlayer;
		actionSelector: #infoFor:inViewer:;
		arguments: (Array with: aScriptOrSlotSymbol with: self);
		label: '' font: (StrikeFont familyName: #ComicBold size: 12);
		color: Color transparent;
		borderWidth: 0;
		actWhen: #buttonDown.
	balloonTextSelector
		ifNotNil:	[aButton balloonTextSelector: balloonTextSelector]
		ifNil:		[aButton setBalloonText: 'Press here to get a menu'].
	^ aButton! !

!CategoryViewer methodsFor: 'entries' stamp: 'sw 9/8/2000 11:01'!
phraseForSlot: slotSpec
	"Return a PhraseTileMorph representing a variable belonging to the player"

	"The slot spec if a tuple with the following structure:
		1	#slot
		2	slot name
		3	balloon help
		4	slot type
		5	#readOnly,# readWrite, or #writeOnly
		6	getter receiver indicator
		7	getter selector
		8	setter receiver indicator
		9	setter selector

	NB:	all are symbols except #3, which is a string"

	| r anArrow slotName getterButton |

	r _ ViewerRow newRow
		color: self color;
		beSticky;
		elementSymbol: (slotName _ slotSpec second);
		centering: #center.

	r addMorphBack: (self slotHeaderFor: slotName).
	r addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"

	r addMorphBack: (self infoButtonFor: slotName).
	r addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"

	r addMorphBack: self tileForSelf bePossessive.
	r addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
	r addMorphBack: (getterButton _ self getterButtonFor: slotName type: slotSpec fourth).
	getterButton setBalloonText: slotSpec third.

	(slotName == #isOverColor)
		ifTrue:
			[self addIsOverColorDetailTo: r]
		ifFalse:
			[(slotSpec fifth == #readOnly) ifFalse:
				[r addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
				r addMorphBack: (anArrow _ self arrowSetterButtonFor: slotName type: slotSpec fourth)].
			r addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"
			(#(colorSees playerSeeingColor copy) includes: slotName) ifFalse:
		 		[r addMorphBack: (self readoutFor: slotName type: slotSpec fourth readOnly: slotSpec fifth getSelector: slotSpec seventh putSelector: slotSpec ninth)]].
	anArrow ifNotNil: [anArrow step].
	^ r! !


!DataMorph methodsFor: 'data' stamp: 'sw 9/8/2000 11:02'!
becomeField
	"Transform the receiver into one that behaves like a card 'field.  Caution: this relates to an exploratory architecture of a couple of years ago and is probably about to be left in the dust.'"

	| aStack slotNameChosen  |
	aStack _ self pasteUpMorph assuredPlayer.

	slotNameChosen _ aStack addSlotNamedLike: self externalName withValue: self valueFromContents.

	self getSelector: (ScriptingSystem getterSelectorFor: slotNameChosen).
	self putSelector: (ScriptingSystem setterSelectorFor: slotNameChosen).
	self target: aStack.
	status _ #field.
	aStack updateAllViewers! !


!Morph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:30'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
		(#basic (
			(slot x 'The x coordinate' number readWrite player getX player setX:)
			(slot y  	'The y coordinate' number readWrite	player 	getY player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' number readWrite player getHeading player setHeading:)
			(command forward: 'Moves the object forward in the direction it is heading' number)
			(command turn: 'Change the heading of the object by the specified amount' number)
			(command beep: 'Make the specified sound' sound)))

		(#'color & border' (
			(slot color 'The color of the object' color readWrite player getColor  player  setColor:)
			(slot colorUnder 'The color under the center of the object' color readOnly player getColorUnder unused  unused )
			(slot borderColor 'The color of the object''s border' color readWrite player getBorderColor player  setBorderColor:)
			(slot borderWidth 'The width of the object''s border' number readWrite player getBorderWidth player setBorderWidth:)))

		(geometry (
			(slot  scaleFactor 'Yeah, the scale factor' number readWrite player getScaleFactor player setScaleFactor:)
			(slot  left   'The left edge, yeah' number readWrite player getLeft  player  setLeft:)
			(slot right  'The right edge, yeah' number readWrite player getRight  player  setRight:)
			(slot  top  'The top edge' number readWrite player getTop  player  setTop:) 
			(slot  bottom  'The bottom edge' number readWrite player getBottom  player  setBottom:) 
			(slot  width  'The width' number readWrite player getWidth  player  setWidth:)
			(slot  height  'The height' number readWrite player getRight  player  setHeight:) 
			(slot x   'The x coordinate' number readWrite player  getX   player setX:)
			(slot y   'The y coordinate' number readWrite player  getY  player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' number readWrite player getHeading  player setHeading:)))

		(miscellaneous (
			(command doMenuItem: 'do the menu item' menu) 
			(command show 'show the guy')
			(command hide 'hide the guy')
			(command wearCostumeOf: 'wear the costume of...' player)
			(command startScript: 'start the given script ticking' string)
			(command stopScript: 'make the given script be "normal"' string)
			(command pauseScript: 'make the given script be "paused"' string)
			(slot copy 'returns a copy of this object' player readOnly player getNewClone	 unused unused)
			(slot elementNumber 'my index in my container' number readWrite player getIndexInOwner player setIndexInOwner:)))

		(motion (
			(slot x 'The x coordinate' number readWrite player getX player setX:)
			(slot y  	'The y coordinate' number readWrite	player 	getY player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' number readWrite player getHeading player setHeading:)
			(command forward: 'Moves the object forward in the direction it is heading' number)
			(slot obtrudes 'whether the object sticks out over its container''s edge' boolean readOnly player getObtrudes unused unused) 
			(command moveToward: 'move toward the given object' player) 
			(command turn: 'Change the heading of the object by the specified amount' number)
			(command bounce: 'bounce off the edge if hit' sound) 
			(command wrap 'wrap off the edge if appropriate') 
			(command followPath 'follow the yellow brick road') 
			(command goToRightOf: 'place this object to the right of another' player)))

		(#'pen use' (
			(slot penColor 'the color of ink used by the pen' color readWrite player getPenColor player setPenColor:) 
			(slot penSize 'the width of the pen' number readWrite player getPenSize player setPenSize:) 
			(slot penDown 'whether the pen is currently down' boolean readWrite player getPenDown player setPenDown:)))

		(#tests (
			(slot isOverColor 'whether any part of the object is over the given color' boolean	readOnly player dummy unused unused) 
			(slot isUnderMouse 'whether the object is under the current mouse position' boolean readOnly	player getIsUnderMouse unused unused)
			(slot colorSees	'whether the given color sees the given color' boolean readOnly	player dummy	unused	unused)
			(slot obtrudes 'whether the object sticks out over its container''s edge' boolean readOnly player getObtrudes unused unused))))! !

!Morph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 16:34'!
additionsToViewerCategory: aCategoryName
	"Answer a list of viewer specs for items to be added to the given category on behalf of the receiver.  Each class in a morph's superclass chain is given the opportunity to add more things"

	self additionsToViewerCategories do:
		[:anAddition | anAddition first == aCategoryName ifTrue: [^ anAddition second]].
	^ #()! !

!Morph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 14:18'!
helpContributions
	"Answer a list of pairs of the form (<symbol> <help message> ) to contribute to the system help dictionary"
	
"NB: Many of the items here are not needed any more since they're specified as part of command definitions now.  Didn't take time to filter them out, though if someone did it would be a service 
  -- sw 9/8/2000 14:17"

	^ #(
		(acceptScript:for:
			'submit the contents of the given script editor as the code defining the given selector')
		(actorState
			'return the ActorState object for the receiver, creating it if necessary')

		(addInstanceVariable
			'start the interaction for adding a new instance variable to the receiver')
		(addPlayerMenuItemsTo:hand:
			'add player-specific menu items to the given menu, on behalf of the given hand.  At present, these are only commands relating to the turtle')
		(addSlotNamedLike:withValue:
			'add a slot with a unique name derived from the first parameter, giving it the second parameter as its initial value')
		(addYesNoToHand
			'Press here to tear off a  TEST/YES/NO unit which you can drop into your script')
		(allScriptEditors
			'answer a list off the extant ScriptEditors for the receiver')
		(amount
			'The amount of displacement')
		(angle	
			'The angular displacement')
		(anonymousScriptEditorFor:
			'answer a new ScriptEditor object to serve as the place for scripting an anonymous (unnamed, unsaved) script for the receiver')
		(append:
			'add an object to this container')
		(assignDecrGetter:setter:amt:
			'evaluate the decrement variant of assignment')
		(assignGetter:setter:amt:
			'evaluate the vanilla variant of assignment')
		(assignIncrGetter:setter:amt:
			'evalute the increment version of assignment')
		(assignMultGetter:setter:amt:
			'evaluate the multiplicative version of assignment')
		(assureEventHandlerRepresentsStatus
			'make certain that the event handler associated with my current costume is set up to conform to my current script-status')
		(assureExternalName
			'If I do not currently have an external name assigned, get one now')
		(assureUniClass
			'make certain that I am a member a uniclass (i.e. a unique subclass); if I am not, create one now and become me into an instance of it')
		(availableCostumeNames
			'answer a list of strings representing the names of all costumes currently available for me')
		(availableCostumesForArrows
			'answer a list of actual, instantiated costumes for me, which can be cycled through as the user hits a next-costume or previous-costume button in a viewer')
		(beep:
			'make the specified sound')
		(borderColor
			'The color of the object''s border')
		(borderWidth
			'The width of the object''s border')
		(bottom
			'My bottom edge, measured downward from the top edge of the world')
		(bounce:
			'If object strayed beyond the boundaries of its container, make it reflect back into it, making the specified noise while doing so.')
		(bounce
			'If object strayed beyond the boundaries of its container, make it reflect back into it')
		(chooseTrigger
'When this script should run.
"normal" means "only when called"')
		(clearTurtleTrails
			'Clear all the pen trails in the interior.')
		(color	
			'The object''s interior color')
		(colorSees
			'Whether a given color in the object is over another given color')
		(colorUnder
			'The color under the center of the object')
		(copy
			'Return a new object that is very much like this one')
		(cursor	
			'The index of the chosen element')
		(deleteCard
			'Delete the current card.')
		(dismiss
			'Click here to dismiss me')
		(elementNumber
			'My element number as seen by my owner')
		(firstPage
			'Go to first page of book')
		(followPath
				'Retrace the path the object has memorized, if any.')
		(forward:
			'Moves the object forward in the direction it is heading') 
		(goto:
			'Go to the specfied book page')
		(goToNextCard
			'Go to the next card')
		(goToPreviousCard
			'Go to the previous card.')
		(goToRightOf:
			'Align the object just to the right of any specified object.')
		(heading
			'Which direction the object is facing.  0 is straight up') 
		(height	
			'The distance between the top and bottom edges of the object')
		(hide
			'Make the object so that it does not display and cannot handle input')
		(initiatePainting	
			'Initiate painting of a new object in the standard playfield.')
		(initiatePaintingIn:
			'Initiate painting of a new object in the given place.')
		(isOverColor
			'Whether any part of this object is directly over the specified color')
		(isUnderMouse
			'Whether any part of this object is beneath the current mouse-cursor position')
		(lastPage
			'Go to the last page of the book.')
		(left
			'My left edge, measured from the left edge of the World')
		(leftRight
			'The horizontal displacement')
		(liftAllPens
			'Lift the pens on all the objects in my interior.')
		(lowerAllPens
			'Lower the pens on all the objects in my interior.')
		(mouseX
			'The x coordinate of the mouse pointer')
		(mouseY
			'The y coordinate of the mouse pointer')
		(moveToward:
			'Move in the direction of another object.')
		(newCard
			'Create a new card.')
		(nextPage
			'Go to next page.')
		(numberAtCursor
			'The number held by the object at the chosen element')
		(objectNameInHalo
			'Object''s name -- To change: click here, edit, hit ENTER')
		(obtrudes
			'Whether any part of the object sticks out beyond its container''s borders')
		(offerScriptorMenu
			'Owner and script name.
Press to get a menu')
		(pauseScript:
			'Make a running script become paused.')
		(penDown
			'Whether the object''s pen is down (true) or up (false)')
		(penColor
			'The color of the object''s pen')
		(penSize	
			'The size of the object''s pen')
		(playerSeeingColorPhrase
			'The player who "sees" a given color')
		(previousPage
			'Go to previous page')

		(show
			'If object was hidden, make it show itself again.')
		(startScript:
			'Make a script start running.')
		(stopScript:
			'Make a script stop running.')
		(top
			'My top edge, measured downward from the top edge of the world')
		(right
			'My right edge, measured from the left edge of the world')
		(roundUpStrays
			'Bring all out-of-container subparts back into view.')
		(scaleFactor
			'The amount by which the object is scaled')
		(stopScript:
			'make the specified script stop running')
		(try
			'Run this command once.')
		(tryMe
			'Click here to run this script once; hold button down to run repeatedly.')
		(turn:				
			'Change the heading of the object by the specified amount')
		(unhideHiddenObjects
			'Unhide all hidden objects.')
		(upDown
			'The vertical displacement')
		(userScript
			'This is a script defined by you.')
		(userSlot
			'This is an instance variable defined by you.  Click here to change its type')
		(valueAtCursor
			'The chosen element')
		(wearCostumeOf:
			'Wear the same kind of costume as the other object')
		(width	
			'The distance between the left and right edges of the object')
		(wrap
			'If object has strayed beond the boundaries of its container, make it reappear from the opposite edge.')
		(x
			'The x coordinate, measured from the left of the container')
		(y
			'The y-coordinate, measured upward from the bottom of the container')

		)
! !


!BookMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:24'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((#'book navigation'
			((command goto: 'go to the given page' player)
			(command nextPage 'go to next page')
			(command previousPage 'go to previous page')
			(command firstPage 'go to first page')
			(command lastPage 'go to last page'))))! !


!GraphMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:25'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(

	(basic (
(slot cursor 	'The current cursor location, wrapped back to the beginning if appropriate' number	 readWrite player getCursor player setCursorWrapped:)

(slot sampleAtCursor	'The sample value at the current cursor location' number readWrite player getSampleAtCursor player setSampleAtCursor:)))

	(sampling
(slot cursor 	'The current cursor location, wrapped back to the beginning if appropriate' number	 readWrite player getCursor player setCursorWrapped:)
(slot sampleAtCursor	'The sample value at the current cursor location' number readWrite player getSampleAtCursor player setSampleAtCursor:)
(slot lastValue 'The last value obtained' number readWrite	player getLastValue player setLastValue:)
(command clear 'Clear the graph of current contents')
(command loadSineWave 'Load a sine wave as the current graph')
(command loadSound: 'Load the specified sound into the current graph' sound)
(command reverse 'Reverse the graph')
(command play 'Play the current graph as a sound')))! !


!JoystickMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:35'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((joystick (
(slot amount 'The amount of displacement' number readOnly player getAmount unused unused)
(slot angle 'The angular displacement' number readOnly player getAngle  unused  unused)
(slot leftRight  'The horizontal displacement' number  readOnly player getLeftRight  unused  unused)
(slot upDown 'The vertical displacement' number  readOnly player getUpDown unused unused))))


! !


!PasteUpMorph methodsFor: 'cursor' stamp: 'sw 9/7/2000 11:43'!
cursor: aNumber
	"for backward compatibility"

	self cursorWrapped: aNumber! !

!PasteUpMorph methodsFor: 'cursor' stamp: 'sw 9/7/2000 11:42'!
cursorWrapped: aNumber
	"Set the cursor to the given number, modulo the number of items I contain. Fractional cursor values are allowed."

	| truncP |
	cursor ~= aNumber ifTrue: 
		[cursor _ self asNumber: aNumber.
		truncP _ cursor truncated.
		truncP > submorphs size ifTrue:
			[submorphs size > 0
				ifTrue: [cursor _ cursor \\ submorphs size]
				ifFalse: [cursor _ 1]].
		truncP < 0 ifTrue: [cursor _ 1].
		self changed]
! !

!PasteUpMorph methodsFor: 'cursor' stamp: 'sw 9/8/2000 16:41'!
numberAtCursor
	"Answer the number represented by the object at my current cursor position"

	| chosenMorph |
	submorphs isEmpty ifTrue: [^ 0].
	chosenMorph _ submorphs at: ((cursor truncated max: 1) min: submorphs size).
	^ chosenMorph getNumericValue
! !

!PasteUpMorph methodsFor: 'cursor' stamp: 'sw 9/8/2000 16:42'!
valueAtCursor
	"Answer the submorph of mine indexed by the value of my 'cursor' slot"

	submorphs isEmpty ifTrue: [^ self presenter standardPlayer costume].
	^ submorphs at: ((cursor truncated max: 1) min: submorphs size)
! !

!PasteUpMorph methodsFor: 'dropping/grabbing' stamp: 'sw 9/1/2000 05:37'!
justDroppedInto: aMorph event: anEvent
	"This message is sent to a dropped morph after it has been dropped on--and been accepted by--a drop-sensitive morph"

	super justDroppedInto: aMorph event: anEvent.
	self isPartsBin ifTrue: [self setPartsBinStatusTo: true]  "gets some things right about the subtle case of dropping a parts bin"
! !

!PasteUpMorph methodsFor: 'menu & halo' stamp: 'sw 9/1/2000 10:23'!
playfieldOptionsMenu
	"Answer an auxiliary menu with options specific to playfields -- too many to be housed in the main menu"

	| aMenu isWorld |
	isWorld _ self isWorldMorph.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addStayUpItem.
	aMenu add: 'save on file...' action: #saveOnFile.
	aMenu add: 'save as SqueakPage at url...' action: #saveOnURL.
	aMenu add: 'update all from resources' action: #updateAllFromResources.
	(self valueOfProperty: #classAndMethod) ifNotNil:
		[aMenu add: 'broadcast as documentation' action: #saveDocPane].
	aMenu add: 'round up strays' action: #roundUpStrays.
	aMenu balloonTextForLastItem:  'Bring back all objects whose current coordinates keep them from being visible, so that at least a portion of each of my interior objects can be seen.'.
	aMenu addLine.
	aMenu add: 'shuffle contents' action: #shuffleSubmorphs.
	aMenu balloonTextForLastItem: 'Rearranges my contents in random order'.
	self griddingOn
		ifTrue: [aMenu add: 'turn gridding off' action: #griddingOnOff.
				aMenu add: (self gridVisible ifTrue: ['hide'] ifFalse: ['show']) , ' grid'
						action: #gridVisibleOnOff.
				aMenu add: 'set grid spacing...' action: #setGridSpec]
		ifFalse: [aMenu add: 'turn gridding on' action: #griddingOnOff].
	aMenu addLine.

	#(	(autoLineLayoutString	toggleAutoLineLayout
			'whether submorphs should automatically be laid out in lines')
		(indicateCursorString	toggleIndicateCursor
			'whether the "current" submorph should be indicated with a dark black border')
		(isPartsBinString		toggleIsPartsBin
			'whether dragging an object from the interior should produce a COPY of the object')
		(isOpenForDragNDropString	toggleDragNDrop
			'whether objects can be dropped into and dragged out of me')
		(mouseOverHalosString	toggleMouseOverHalos
			'whether objects should put up halos when the mouse is over them')
		(autoExpansionString	toggleAutomaticPhraseExpansion
			'whether tile phrases, dropped on me, should automatically sprout Scriptors around them')
		(originAtCenterString	toggleOriginAtCenter
			'whether the cartesian origin of the playfield should be at its lower-left corner or at the center of the playfield')
		(showThumbnailString	toggleAlwaysShowThumbnail
			'whether large objects should be represented by thumbnail miniatures of themselves')) do:

			[:triplet |
				(isWorld and: [#(toggleAutoLineLayout toggleIndicateCursor toggleIsPartsBin toggleAlwaysShowThumbnail) includes: triplet second]) ifFalse:
					[aMenu addUpdating: triplet first action: triplet second.
					aMenu balloonTextForLastItem: triplet third]]. 

	isWorld ifFalse:
		[aMenu add: 'set thumbnail height...' action: #setThumbnailHeight.
		aMenu balloonTextForLastItem: 'if currently showing thumbnails governs the standard height for them'].

	aMenu addUpdating: #autoViewingString action: #toggleAutomaticViewing.
	aMenu balloonTextForLastItem:  'governs whether, when an object is touched inside me, a viewer should automatically be launched for it.'.

	isWorld ifFalse:
		[aMenu add: 'behave like a Holder' action: #becomeLikeAHolder.
		aMenu balloonTextForLastItem: 'Set properties to make this object nicely set up to hold frames of a scripted animation.'].

	self backgroundSketch ifNotNil:
		[aMenu add: 'delete background painting' action: #deleteBackgroundPainting.
		aMenu balloonTextForLastItem: 'delete the graphic that forms the background for this me.'].
	presenter ifNil:
		[aMenu add: 'make detachable' action: #makeDetachable.
		aMenu balloonTextForLastItem: 'Allow this area to be separately governed by its own controls.'].

	aMenu addLine.
	aMenu add: 'use standard texture' action: #setStandardTexture.
	aMenu balloonTextForLastItem: 'use a pale yellow-and-blue background texture here.'.
	aMenu add: 'make graph paper...' action: #makeGraphPaper.
	aMenu balloonTextForLastItem: 'Design your own graph paper and use it as the background texture here.'.
	aMenu addTitle: 'playfield options...'.

	^ aMenu
! !

!PasteUpMorph methodsFor: 'scripting' stamp: 'sw 9/8/2000 16:40'!
emptyPermanentScriptorFrom: aPlaceHoldingMorph
	"Answer a new empty permanent scriptor derived from info deftly secreted in aPlaceHoldingMorph.  Good grief"

	| aScriptor aPlayer |
	aPlayer _ aPlaceHoldingMorph valueOfProperty: #player.
	aPlayer assureUniClass.
	aScriptor _  aPlayer permanentScriptEditorFor: nil.
	aScriptor position: (self primaryHand position - (10 @ 10)).
	aPlayer updateAllViewersAndForceToShow: #scripts.
	^ aScriptor! !


!PasteUpMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 18:38'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(

(playfield (
(command initiatePainting 'Initiate painting of a new object in the standard playfield.')
(slot mouseX 'The x coordinate of the mouse pointer' number readWrite player getMouseX  unused unused)
(slot mouseY 'The y coordinate of the mouse pointer' number readWrite player getMouseY  unused unused)
(command roundUpStrays 'Bring all out-of-container subparts back into view.')
(slot numberAtCursor 'the number at the cursor' number readWrite player getNumberAtCursor player setNumberAtCursor: )
(slot playerAtCursor 'the object currently at the cursor' player readWrite player getValueAtCursor  unused unused)
(command unhideHiddenObjects 'Unhide all hidden objects.')))

(collections (
(slot cursor 'The index of the chosen element' number readWrite player getCursor player setCursorWrapped:)
(slot playerAtCursor 'the object currently at the cursor' player readWrite player getValueAtCursor  unused unused)
(slot numberAtCursor 'the number at the cursor' number readWrite player getNumberAtCursor player setNumberAtCursor: )
(command removeAll 'Remove all elements from the playfield')
(command shuffleContents 'Shuffle the contents of the playfield')
(command append: 'Add the object to my content' player)))

(#'card/stack' (
(command goToNextCard 'Go to the next card')
(command goToPreviousCard  'Go to the previous card')
(command deleteCard 'Delete the current card')
(command newCard 'Create a new card')))

(#'pen trails' (
(command liftAllPens 'Lift the pens on all the objects in my interior.')
(command lowerAllPens  'Lower the pens on all the objects in my interior.')
(command clearTurtleTrails 'Clear all the pen trails in the interior.'))))
! !


!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/8/2000 16:42'!
bePossessive
	"No way for doubly possessive stuff to work at present, so we just catch and swallow this request here."

	self flag: #deferred! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/1/2000 08:41'!
operatorTile
	"Answer the submorph which comprises the operator tile of the receiver.  Ouch!!"

	^ submorphs second! !

!PhraseTileMorph methodsFor: 'tiles from method' stamp: 'sw 9/8/2000 09:44'!
tilesFrom: msgNode in: aScriptor
	"Construct a single line of tiles from a MessageNode of a parse tree.  For a single message send."

	| tile pm sel instVar suff argNode selType |
	self flag: #noteToTed.  "Ted: this is your code from 8/6/99, which is now broken before it could ever get used.  The method #scriptInfoFor: which it formerly called is gone; I've included its old content at the end in comments as a pointer.  sw 9/8/2000 09:44"

	sel _ msgNode selector key.
	(sel beginsWith: 'assign') ifTrue: [	"assignment"
		instVar _ msgNode arguments first literalValue.
		instVar _ (instVar copyFrom: 4 to: instVar size) withFirstCharacterDownshifted.
		pm _ (CategoryViewer new) scriptedPlayer: aScriptor playerScripted; 
			makeSetter: nil from: aScriptor playerScripted costume 
			forPart: (Array with: instVar with: #number).	"makes a new Phrase"
		suff _ (sel findTokens: ':') first.
		suff _ (suff copyFrom: 7 to: suff size-6), ':'.		": Incr: Decr: Mult:"
		pm submorphs second setAssignmentSuffix: suff.
		pm submorphs third delete.
		tile _ TilePadMorph new tilesFrom: msgNode arguments last in: aScriptor.
		pm addMorphBack: tile.
		^ pm].

	(sel beginsWith: 'get') ifTrue: [	"getter"
		instVar _ (sel copyFrom: 4 to: sel size) withFirstCharacterDownshifted.
		pm _ (CategoryViewer new) scriptedPlayer: aScriptor playerScripted; 
			makeGetter: nil from: aScriptor playerScripted costume 
			forPart: (Array with: instVar with: #number).	"makes a new Phrase"
		^ pm].

	(#(ifTrue:ifFalse: ifFalse: ifTrue:) includes: sel) ifTrue: [
		^ CompoundTileMorph new tilesFrom: msgNode in: aScriptor].

	self addMorphBack: (TilePadMorph new tilesFrom: msgNode receiver in: aScriptor).
	self addMorphBack: (TileMorph new selectorTile: msgNode in: aScriptor).	"selector"
	(aScriptor playerScripted elementTypeFor: sel) == #systemScript ifTrue: [
		selType _ (aScriptor playerScripted phraseSpecFor: 
			(Array with: #command with: sel)) last].
	msgNode arguments size > 0 ifTrue: [
		argNode _ msgNode arguments last.
		tile _ TileMorph new tilesFrom: argNode type: selType in: aScriptor.
		self addMorphBack: tile].


"Old code from Player, now superseded by an entirely new mechanism...
phraseSpecFor: aPair
	| info prefix |
	info _ (prefix _ aPair first) == #slot
		ifTrue:
			[ScriptingSystem slotInfoFor: aPair second]
		ifFalse:
			[ScriptingSystem scriptInfoFor: aPair second].
	^ (Array with: prefix), info"
! !


!Player methodsFor: 'slots-kernel' stamp: 'sw 9/8/2000 14:38'!
categories
	"Answer a list of categories appropriate to the the receiver and its costumes"

	| aList |
	(self hasCostumeThatIsAWorld)
		ifTrue:	[^ self categoriesForWorld].

	aList _ OrderedCollection new.
	self slotNames size > 0 ifTrue:
		[aList add: #'instance variables'].
	self class scripts size > 0 ifTrue:
		[aList add: #scripts].
	aList addAll: costume categoriesForViewer.
	^ aList! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/8/2000 17:02'!
categoriesForWorld
	"Answer the list of categories given that the receiver is the Player representing a World"

	| aList |
	aList _ #(#'color & border' #'pen trails' playfield) asOrderedCollection.
	self class scripts size > 0 ifTrue:
		[aList addFirst: #scripts].
	self slotNames size > 0 ifTrue:
		[aList addFirst: #'instance variables'].

	^ aList! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/8/2000 10:21'!
tilePhrasesSpecsForCategory: aCategory
	"Return an array of slot and script names and info for use in a viewer on the receiver.  These can be of two flavors - command and slot."

	| categorySymbol |
	categorySymbol _ aCategory asSymbol.

	(categorySymbol == #'instance variables') ifTrue:		"user-defined instance variables"
		[^ self tilePhrasesSpecsInstanceVariablesCategory].

	(categorySymbol == #scripts) ifTrue:						"user-defined scripts"
		[^ self tileScriptCommands].

	^ (self usablePhraseSpecsIn: (costume categoryElementsFor: categorySymbol))   "all others"


"The format of the specs is illustrated by:
	(slot heading 'the direction'	number	 readWrite player getHeading player setHeading:)
	(command wearCostumeOf: 'change appearance to look like' player)"

! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/8/2000 17:08'!
tilePhrasesSpecsInstanceVariablesCategory
	"Return an array of slot names and info for use in a viewer on the receiver to represent the instance-variables category"

	| nameString |
	^ self slotNames collect: [:aName | nameString _ aName asString capitalized.

		{#slot.				"NB: Brace construct here"
		aName asSymbol.					"name"
		'a user-defined instance variable'.	"balloon help"
		(self typeForSlot: aName asSymbol).	"type"
		#readWrite.							"r/w"
		#player.							"receiver of get  selector"
		('get', nameString) asSymbol.		"get selector"
		#player.							"receiver of set selector"
		('set', nameString, ':') asSymbol}]	"set selector"
! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/8/2000 11:20'!
usablePhraseSpecsIn: aListOfTuples
	"Filter the list given by aListOfTuples, to remove items inappropriate to the receiver"

	self hasCostumeThatIsAWorld ifTrue:
		[^ aListOfTuples select: [:tuple |
			#(beep doMenuItem color startScript: stopScript: pauseScript: liftAllPens lowerAllPens clearTurtleTrails initiatePainting cursor valueAtCursor mouseX mouseY roundUpStrays unhideHiddenObjects) includes: tuple second]].

	self hasAnyBorderedCostumes ifTrue: [^ aListOfTuples].

	^ self hasOnlySketchCostumes
		ifTrue:
			[aListOfTuples select: [:tuple | (#(color borderColor borderWidth) includes: tuple second) not]]
		ifFalse:
			[aListOfTuples select: [:tuple | (#(borderColor borderWidth) includes: tuple second) not]]
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:32'!
getCharacters
	"Answer the characters in my costume, likely a TextMorph"

	^ costume getCharacters! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:25'!
getCursor
	"Obtain the cursor setting from the receiver's costume"

	^ costume renderedMorph cursor
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:25'!
getCursorWrapped
	"maintained for backward compatibility only, for preexisting etoys"

	^ self getCursor
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:34'!
getFirstCharacter
	"Answer a string consisting of the first character in my costume, likely itself a TextMorph"

	^ costume getFirstCharacter! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:35'!
getIndexInOwner
	"Answer my costume's index in its owner"

	^ costume getIndexInOwner! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:35'!
getNewClone
	"Answer a new player of the same class as the receiver, with a costume much like mine"

	^ costume usableSiblingInstance player
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:36'!
getNumberAtCursor
	"Answer the number borne by the object at my costume's current cursor position"

	| renderedMorph aCostume |
	aCostume _ self costume.
	((renderedMorph _ aCostume renderedMorph) respondsTo: #valueAtCursor:) ifTrue: [^ renderedMorph valueAtCursor getNumericValue]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:36'!
getNumericValue
	"Answer the numeric value contained in my costume"

	^ costume getNumericValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 12:21'!
handTheUserACopy
	"Called from the user-interface: hand the user a copy"

	^ costume currentHand attachMorph: self getNewClone costume! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:36'!
setCharacters: amt
	"Set my costume's characters as indicated"

	costume setCharacters: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 11:16'!
setCursor: aNumber
	"Set my costume's cursor to the given number"

	costume renderedMorph cursorWrapped: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:29'!
setCursorWrapped: aNumber
	"maintained for backward compatibility with existing etoys"

	^ self setCursor: aNumber
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:36'!
setFirstCharacter: aChar
	"Set my costume's first character to the indicated one"

	costume setFirstCharacter: aChar! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 11:18'!
setIndexInOwner: amt
	"Move my costume to a different z-position within its container.  This is primarily in service of auto-line-layout views in which the z-ordering determines the left/right and top/bottom placement."

	costume setIndexInOwner: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:37'!
setNumberAtCursor: aNumber
	"Place the given number into the morph residing at my costume's current cursor position"

	| renderedMorph aCostume |
	aCostume _ self costume.
	((renderedMorph _ aCostume renderedMorph) respondsTo: #valueAtCursor:) ifTrue: [^ renderedMorph valueAtCursor setNumericValue: aNumber]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/7/2000 18:21'!
commandPhraseFor: commandSpec inViewer: aViewer
	"Translate commandSpec into a PhraseTileMorph.  Put appropriate balloon help into the phrase"

	| aRow resultType cmd names argType argTile selfTile aPhrase balloonTextSelector |
	names _ self class namedTileScriptSelectors.

	resultType _ (commandSpec at: 1).
	cmd _ (commandSpec at: 2).
	commandSpec size = 3
		ifTrue:
			[aPhrase _ PhraseTileMorph new setOperator: cmd
				type: resultType
				rcvrType: #player]
		ifFalse: "commandSpec size is four"
			[argType _ commandSpec at: 4.
			aPhrase _ PhraseTileMorph new setOperator: cmd
				type: resultType
				rcvrType: #player
				argType: argType.
			argTile _ self tileForArgType: argType inViewer: aViewer.
			argTile position: aPhrase lastSubmorph position.
			aPhrase lastSubmorph addMorph: argTile].
	aPhrase setBalloonText: commandSpec third.

	(names includes: cmd) ifTrue: [aPhrase userScriptSelector: cmd].

	(self slotInfo includesKey: cmd)
		ifTrue: [balloonTextSelector _ #userSlot].

	(self belongsToUniClass and:
			[self class includesSelector: cmd])
		ifTrue: [balloonTextSelector _ #userScript].

	aPhrase operatorTile balloonTextSelector: (balloonTextSelector ifNil: [cmd]).
	aPhrase markAsPartsDonor.

	selfTile _ aViewer tileForSelf.
	selfTile position: aPhrase firstSubmorph position.
	aPhrase firstSubmorph addMorph: selfTile.

	aRow _ ViewerRow newRow borderWidth: 0; color: aViewer color.
	aRow elementSymbol: cmd asSymbol.

	aRow addMorphBack: (ScriptingSystem tryButtonFor: aPhrase).
	aRow addMorphBack: (Morph new extent: 4@2; beTransparent).
	aRow addMorphBack: (aViewer infoButtonFor: cmd).
	aRow addMorphBack: aPhrase.
	aRow beSticky; disableDragNDrop.

	^ aRow! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:03'!
elementTypeFor: aStringOrSymbol
	"Answer whether aStringOrSymbol is best characterized as a #systemSlot, #systemScript, #userSlot, or #userScript"

	| aSymbol |
	aSymbol _ aStringOrSymbol asSymbol.
	(ScriptingSystem typeForSystemSlotNamed: aSymbol) ifNotNil: [^ #systemSlot].
	(ScriptingSystem isSystemScriptName: aSymbol) ifTrue: [^ #systemScript].
	(self slotInfo includesKey: aSymbol) ifTrue: [^ #userSlot].
	^ #userScript! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:03'!
expungeEmptyScripts
	"Track down and destroy -- and destroy screen artifacts relating to -- all scripts belonging to the receiver that have no lines of code in them"

	| any |
	any _ false.
	self class namedTileScriptSelectors do:
		[:aSel |
			(self isEmptyTileScript: aSel)
				ifTrue:
					[any _ true.
					self removeScriptWithoutUpdatingViewers: aSel]].
	any ifTrue: [self updateAllViewersAndForceToShow: #scripts]
			
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:04'!
expungeEmptyUnRenamedScripts
	"Track down and destroy -- and destroy screen artifacts relating to -- all scripts belonging to the receiver that have not been named and that have no lines of code in them"

	| any |
	any _ false.
	self class namedTileScriptSelectors do:
		[:aSel |
			(self isExpendableScript: aSel)
				ifTrue:
					[any _ true.
					self removeScriptWithoutUpdatingViewers: aSel]].
	any ifTrue:
		[self updateAllViewersAndForceToShow: #scripts]
			
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:05'!
permanentScriptEditorFor: aPhraseTileMorph
	"Sprout a scriptor around aPhraseTileMorph, thus making a new script"

	| aScriptEditor aUserScript |
	self isFlagshipForClass ifFalse:
		[^ aPhraseTileMorph].

	aUserScript _ self class permanentUserScriptFor: self unusedScriptName player: self.
	aScriptEditor _ aUserScript instantiatedScriptEditor.
	aPhraseTileMorph ifNotNil:
		[aScriptEditor phrase: aPhraseTileMorph].
	aScriptEditor install.
	self updateAllViewersAndForceToShow: #scripts.
	^ aScriptEditor! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:06'!
removeScript: aSymbol fromWorld: aWorld
	"Remove the given script, and get the display right in aWorld"

	self removeScriptWithoutUpdatingViewers: aSymbol fromWorld: aWorld.
	self updateAllViewersAndForceToShow: #scripts
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:07'!
slotInfoButtonHitFor: aSlotName inViewer: aViewer
	"The user made a gesture asking for info/menu relating"

	| aMenu slotSym aType |
	slotSym _ aSlotName asSymbol.
	aType _ self typeForSlot: aSlotName asSymbol.
	aMenu _ MenuMorph new defaultTarget: self.
	(#(colorSees copy getNewClone) includes: slotSym) ifFalse:
		[aMenu add: 'simple watcher' selector: #tearOffWatcherFor: argument: slotSym].
	(#(copy getNewClone) includes: slotSym) ifTrue:
		[aMenu add: 'give me a copy now' action: #handTheUserACopy].

	aType == #number "later others" ifTrue:
		[aMenu add: 'detailed watcher' selector: #tearOffFancyWatcherFor: argument: slotSym].

	(self slotInfo includesKey: slotSym)
		ifTrue:  "User slot"
			[aMenu add: 'change data type' selector: #chooseSlotTypeFor: argument: slotSym.
			aType == #number ifTrue:
				[aMenu add: 'decimal places...' selector: #setPrecisionFor: argument: slotSym].
			aMenu add: 'remove "', aSlotName, '"' selector: #removeSlotNamed: argument: slotSym.
			aMenu add: 'rename  "', aSlotName, '"' selector: #renameSlot: argument: slotSym].
	aMenu items size == 0 ifTrue:
		[aMenu add: 'ok' action: #yourself].
	aMenu addTitle: (aSlotName asString, ' (', aType, ')').
	aMenu popUpForHand: aViewer primaryHand in: aViewer world! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 11:19'!
tilePhrasesForCategory: aCategory inViewer: aViewer
	"Return an array of phrases for the category"

	^ (self tilePhrasesSpecsForCategory: aCategory) collect:
		[:aSpec |
			aSpec first == #slot
				ifTrue:
					[aViewer phraseForSlot: aSpec]
				ifFalse:
					[self commandPhraseFor: aSpec inViewer: aViewer]]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 14:18'!
tileScriptCommands
	"Return a list of typed-command arrays of the form:
		#command <selector> <balloon help>"

	self flag: #deferred.  "'need way for user to save own help"
	^  self class namedTileScriptSelectors asSortedArray collect:
			[:sel | Array with: #command with: sel with: 'a user-written script']
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/1/2000 10:33'!
append: aPlayer
	"Add aPlayer to the list of objects logically 'within' me.  This is visually represented by its morph becoming my morph's last submorph"

	| aCostume |
	(aPlayer == nil or: [aPlayer == self]) ifTrue: [^ self].
	((aCostume _ self costume topRendererOrSelf) isKindOf: PasteUpMorph)  ifFalse: [^ self].
	aCostume addMorphNearBack:  aPlayer costume.
	self setCursor: (aCostume submorphs indexOf: aPlayer costume)! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/7/2000 12:52'!
getObtrudes
	"Answer whether the receiver's costume obtrudes beyond the bounds of its container"

	| aCostume |
	(aCostume _ self costume) ifNil: [^ false].
	^ aCostume obtrudesBeyondContainer
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/8/2000 17:05'!
removeAll
	"Remove all the elements from my companion morph's collection.  This is destructive!!"

	costume ifNotNil: [costume removeAllMorphs]! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/8/2000 18:39'!
shuffleContents
	"Tell my costume to rearrange its submorphs randomly"

	costume shuffleSubmorphs! !

!Player methodsFor: 'misc' stamp: 'sw 9/8/2000 21:42'!
defaultFloatPrecisionFor: aGetSelector
	"Answer the float position to use in conjunction with a readout for aGetSelector"

	| aSlotName |
	(#(getCursor getNumericValue getNumberAtCursor getCursorWrapped) includes: aGetSelector)
		ifTrue:
			[^ 0.01].
	aSlotName _ Utilities inherentSelectorForGetter: aGetSelector.
	((self elementTypeFor: aSlotName) == #userSlot)
		ifTrue:
			[^ (self slotInfoAt: aSlotName) floatPrecision].

	^ 1! !


!Preferences class methodsFor: 'help' stamp: 'sw 9/8/2000 11:21'!
helpMsgsQThroughZ
	"Answer a list of <symbol> <string> pairs that give the balloon help corresponding to various symbols"

	^ #(	
(reverseWindowStagger
	'If true, a reverse-stagger strategy  is used for determining where newly launched windows will be placed; if false, a direct- stagger strategy is used.')

(roundedMenuCorners
	'Whether morphic menus should have rounded corners')

(roundedWindowCorners
	'Governs whether morphic system windows should have rounded corners')

(scrollBarsNarrow
		'If true, morphic scrollbars will be narrow.')
(scrollBarsOnRight
		'If true, morphic scrollbars in subsequently opened windows will appear on the right side of their pane.')
(scrollBarsWithoutMenuButton
		'If true, morphic scrollbars in subsequently opened windows will not include a menu button.')

(smartUpdating
	'If true, then morphic tools such as browsers and inspectors will keep their contents up to date automatically, so that if something changes anywhere, the change will be reflected everywhere.')

(soundQuickStart
	'If true, attempt to start playing sounds using optional "quick start"')

(soundsEnabled
	'If false, all sound playing is disabled')

(systemWindowEmbedOK
	'Determines whether, in Morphic, when a SystemWindow is dropped onto a willing receptor it should be deposited into that receptor.')

(timeStampsInMenuTitles
	'If true, then the author''s timestamp is displayed as the menu title of any message list; if false, no author''s timestamps are shown')

(thoroughSenders
	'If true, then ''senders'' browsers will dive inside structured literals in their search')

(twentyFourHourFileStamps
	'If #changeSetVersionNumbers is false, this preference determines whether the date/time suffix used with changeset fileouts is based on a 24-hr clock or a 12-hr clock.')

(uniformWindowColors
	'If true, then all standard windows are given the same color rather than their customized window-type-specific colors')

(unlimitedPaintArea
	'If true, the painting area for a new drawing will not be limited in size; if false, a reasonable
limit will be applied, in an attempt to hold down memory and time price.')

(updateRemoveSequenceNum
	'If true, then remove the leading sequence number from the filename before automatically saving a local copy of any update loaded.')

(updateSavesFile
	'If true, then when an update is loaded from the server, a copy of it will automatically be saved on a local file as well.')

(useGlobalFlaps
	'If true, then flaps are shown along the edges of Morphic projects.')

(viewersInFlaps
	'If true, viewers are projected into flaps along the right edge of the screen')

(warnIfNoChangesFile
	'If true, then you will be warned, whenever you start up, if no changes file
can be found')

(warnIfNoSourcesFile 
	'If true, then you will be warned, whenever you start up, if no sources file can be found')


			) 
! !


!ReferenceMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:29'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((paintbox ((command makeNewDrawingIn: 'make a new drawing in the specified playfield' player))))



! !


!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 9/8/2000 17:09'!
actuallyDestroyScript
	"Carry out the actual destruction of the associated script."

	| aHandler |
	self delete.
	playerScripted class removeScriptNamed: scriptName.
	playerScripted actorState instantiatedUserScriptsDictionary removeKey: scriptName.
		"not quite enough yet in the multiple-instance case..."
	(aHandler _ playerScripted costume renderedMorph eventHandler) ifNotNil:
		[aHandler forgetDispatchesTo: scriptName].
	playerScripted updateAllViewersAndForceToShow: #scripts! !


!SimpleSliderMorph methodsFor: 'private' stamp: 'sw 9/1/2000 10:38'!
getNumericValue
	"Answer the numeric value of the receiver"

	^ self getScaledValue! !

!SimpleSliderMorph methodsFor: 'private' stamp: 'sw 9/1/2000 10:38'!
setNumericValue: aValue
	"Set the numeric value of the receiver to be as indicated"

	^ self setScaledValue: aValue! !


!SimpleSliderMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:29'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
(slider (

(slot numericValue 'A number representing the current position of the knob.' number readWrite player getNumericValue player setNumericValue:)

(slot minVal	 'The number represented when the knob is at the left or top of the slider; the smallest value returned by the slider.' number readWrite	player getMinVal player setMinVal:)

(slot maxVal 'The number represented when the knob is at the right or bottom of the slider; the largest value returned by the slider.' number readWrite	player getMaxVal player setMaxVal:)

(slot descending 'Tells whether the smallest value is at the top/left (descending = false) or at the bottom/right (descending = true)' boolean readWrite player getDescending player setDescending:)

(slot truncate 'If true, only whole numbers are used as values; if false, fractional values are allowed.' boolean readWrite	player getTruncate player setTruncate:)

(slot color 'The color of the slider' color readWrite player getColor  player  setColor:)

(slot knobColor 'The color of the slider' color readWrite player getKnobColor player setKnobColor:)
(slot  width  'The width' number readWrite player getWidth  player  setWidth:)
(slot  height  'The height' number readWrite player getRight  player  setHeight:)))

(basic	(
(slot numericValue 'A number representing the current position of the knob.' number readWrite player getNumericValue player setNumericValue:))))

! !


!SpeakerMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:28'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^  #((speaker

((slot conePosition  'the position of the speaker cone' number readOnly player getConePosition player setConePosition:))))



! !


!StandardScriptingSystem methodsFor: 'help dictionary' stamp: 'sw 9/8/2000 17:19'!
initReservedScriptNames
	"Initialize the ReservedScriptNames class var, by polling the classes that have defined viewer-category information"

	| additions aSet |
	"ScriptingSystem initReservedScriptNames"
	aSet _ IdentitySet new.
	Morph withAllSubclasses do:
		[:aClass |
			(aClass class includesSelector: #additionsToViewerCategories) ifTrue:
				[additions _ aClass additionsToViewerCategories.
				additions do:
					[:catNameAndInfo |
						catNameAndInfo second do:
							[:spec | (spec first == #command) ifTrue:
								[aSet add: spec second]]]]].

	ReservedScriptNames _ aSet! !

!StandardScriptingSystem methodsFor: 'utilities' stamp: 'sw 9/7/2000 14:08'!
getterSelectorFor: identifier
	"Some idiosyncratic substitutions here..."

	| aSymbol |
	(aSymbol _ identifier asSymbol) == #isOverColor: ifTrue: [^ #seesColor:].
	aSymbol == #copy ifTrue: [^ #getNewClone].

	^ Utilities getterSelectorFor: aSymbol! !

!StandardScriptingSystem methodsFor: 'utilities' stamp: 'sw 9/1/2000 08:58'!
holderWithAlphabet
	"Answer a fully instantiated Holder that has submorphs that represent the letters of the uppercase alphabet, with each one having an 'index' slot which bears the letter's index in the alphabet -- 1 for A, 2 for B, etc."

	| aMorph aPlayer newMorph oneCharString aContainer aWrapper |

	"self currentHand attachMorph: ScriptingSystem holderWithAlphabet"

	aContainer _ self prototypicalHolder useRoundedCorners.
	aContainer borderColor: Color blue lighter.

	aWrapper _ AlignmentMorph new hResizing: #shrinkWrap; vResizing: #shrinkWrap; inset: 0.
	aWrapper addMorphBack: (aMorph _ TextMorph new contents: 'A').
	aMorph beAllFont: ((TextStyle named: #ComicBold) fontOfSize: 24).
	aMorph width: 14; lock.
	aWrapper beTransparent; setNameTo: 'A'.
	aPlayer _ aWrapper assuredPlayer.
	aPlayer addInstanceVariableNamed: #index type: #number value: 1.
	aContainer addMorphBack: aWrapper.
	2 to: 26 do:
		[:anIndex |
			newMorph _ aWrapper usableSiblingInstance.
			newMorph player perform: #setIndex: with: anIndex.
			newMorph firstSubmorph contents: (oneCharString _ ($A asciiValue + anIndex - 1) asCharacter asString).
			newMorph setNameTo: oneCharString.

			aContainer addMorphBack: newMorph].

	aContainer setNameTo: 'alphabet'.
	aContainer isPartsBin: true.
	aContainer indicateCursor: false.
	aContainer color: (Color r: 0.839 g: 1.0 b: 1.0).  "Color fromUser"
	^ aContainer! !

!StandardScriptingSystem methodsFor: 'utilities' stamp: 'sw 9/1/2000 10:39'!
wordingForOperator: aString
	"Answer the wording to be seen by the user for the given operator symbol/string"

	| toTest |
	"StandardScriptingSystem initialize"
	toTest _ aString asString.
	#(	(append:				'append')
		(beep:					'make sound')
		(bounce:				'bounce')
		(clearTurtleTrails		'clear pen trails')
		(doMenuItem:			'do menu item')
		(forward:				'forward by')
		(moveToward:			'move toward')
		(goToRightOf:			'align after')
		(isDivisibleBy:			'is divisible by')
		(liftAllPens				'lift all pens')
		(lowerAllPens			'lower all pens')
		(pauseScript:			'pause script')
		(max:					'max')
		(min:					'min')
		(seesColor:				'is over color')
		(makeNewDrawingIn:	'start painting in')
		(startScript:				'start script')
		(stopProgramatically	'stop')
		(stopScript:				'stop script')
		(turn:					'turn by')
		(wearCostumeOf:		'look like'))

	do:
		[:pair | toTest = pair first ifTrue: [^ pair second]].

	^ toTest! !

!StandardScriptingSystem methodsFor: 'universal slots & scripts' stamp: 'sw 9/8/2000 17:13'!
initializeSystemSlotDictionary
	"Initialize a structure that gives names and types for all the slots and pseudo-slots defined in the current scripting vocabulary.  The keys to this dictionary are slot-name-symbols, and the values are value-type-symbols"

	| aDictionary additions |
	"ScriptingSystem initializeSystemSlotDictionary"
	aDictionary _ IdentityDictionary new.
	Morph withAllSubclasses do:
		[:aClass |
			(aClass class includesSelector: #additionsToViewerCategories) ifTrue:
				[additions _ aClass additionsToViewerCategories.
				additions do:
					[:catNameAndInfo |
						catNameAndInfo second do:
							[:spec | (spec first == #slot) ifTrue:
								[aDictionary at: spec second put: spec fourth]]]]].

	SystemSlotDictionary _ aDictionary! !

!StandardScriptingSystem methodsFor: 'universal slots & scripts' stamp: 'sw 9/8/2000 17:10'!
isSystemScriptName: aName
	"Answer whether the given name is a reserved script name"

	^ ReservedScriptNames includes: aName asSymbol! !


!StandardScriptingSystem class methodsFor: 'class initialization' stamp: 'sw 9/8/2000 17:19'!
initialize
	"Initialize the scripting system.  Sometimes this method is vacuously changed just to get it in a changeset so that its invocation will occur as part of an update"

	(Smalltalk at: #ScriptingSystem ifAbsent: [nil]) ifNil:
		[Smalltalk at: #ScriptingSystem put: self new].

	ScriptingSystem
		initReservedScriptNames;
		initializeSystemSlotDictionary;
		initializeHelpStrings

"StandardScriptingSystem initialize"! !


!StandardViewer methodsFor: 'categories' stamp: 'sw 9/8/2000 17:10'!
likelyCategoryToShow
	"Choose a category to show based on what's already showing and on some predefined heuristics"

	| possible |
	possible _ scriptedPlayer categories asOrderedCollection.
	self categoryMorphs do:
		[:m | possible remove: m currentCategory ifAbsent: []].

	((possible includes: #'instance variables') and: [scriptedPlayer hasUserDefinedSlots])
		ifTrue:	[^ #'instance variables'].

	((possible includes: #scripts) and: [scriptedPlayer hasUserDefinedScripts])
		ifTrue:	[^ #'scripts'].

	#(basic #'color & border') do:
		[:preferred | (possible includes: preferred) ifTrue: [^ preferred]].
	^ possible size > 0
		ifTrue:
			[possible first]
		ifFalse:
			[#basic]! !


!String methodsFor: 'converting' stamp: 'sw 9/1/2000 10:39'!
asCharacter
	"Answer the receiver's first character, or '' if none.  Idiosyncratic, provisional."

	^ self size > 0 ifTrue: [self first] ifFalse: [$]! !

!String methodsFor: 'converting' stamp: 'sw 9/1/2000 10:40'!
asInteger 
	"Answer the Integer created by interpreting the receiver as the string representation of an integer.  Answer nil if no digits, else find the first digit and then all consecutive digits after that"

	| startPosition tail endPosition |
	startPosition _ self findFirst: [:ch | ch isDigit].
	startPosition == 0 ifTrue: [^ nil].
	tail _ self copyFrom: startPosition to: self size.
	endPosition _ tail findFirst: [:ch | ch isDigit not].
	endPosition == 0 ifTrue: [endPosition _ tail size + 1].
	^ Number readFromString: (tail copyFrom: 1 to: endPosition - 1)

"
'1796exportFixes-tkMX' asInteger
'1848recentLogFile-sw'  asInteger
'donald' asInteger
'abc234def567' asInteger
"! !


!TextMorph methodsFor: 'accessing' stamp: 'sw 8/28/2000 09:23'!
getCharacters
	"obtain a string value from the receiver"

	^ text string copy! !

!TextMorph methodsFor: 'accessing' stamp: 'sw 9/1/2000 10:43'!
getFirstCharacter
	"obtain the first character from the receiver if it is empty, return a black dot"

	| aString |
	^ (aString _ text string) size > 0 ifTrue: [aString first] ifFalse: ['']! !

!TextMorph methodsFor: 'accessing' stamp: 'sw 8/28/2000 11:09'!
getNumericValue
	"obtain a numeric value from the receiver; if no digits, return zero"

	^ [text string asNumber] ifError: [:a :b | ^ 0]! !

!TextMorph methodsFor: 'accessing' stamp: 'sw 9/1/2000 10:43'!
setCharacters: chars
	"obtain a string value from the receiver"

	(self getCharacters = chars) ifFalse:
		[self newContents: chars]! !

!TextMorph methodsFor: 'accessing' stamp: 'sw 9/1/2000 10:43'!
setFirstCharacter: source
	"Set the first character of the receiver as indicated"

	| aChar chars |
	aChar _ source asCharacter.
	(chars _ self getCharacters) size > 0 
		ifFalse:
			[self newContents: (String with: aChar)]
		ifTrue:
			[(chars first = aChar) ifFalse:
				[self newContents: (String streamContents:
					[:aStream |
						aStream nextPut: aChar.
						aStream nextPutAll: (chars copyFrom: 2 to: chars size)])]]! !

!TextMorph methodsFor: 'accessing' stamp: 'sw 9/1/2000 10:44'!
setNumericValue: aValue
	"Set the contents of the receiver to be a string obtained from aValue"

	self newContents: aValue asString! !


!TextMorph class methodsFor: 'scripting' stamp: 'sw 9/8/2000 15:28'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
(text (
(slot characters	'The characters in my contents' string	readWrite player getCharacters player setCharacters:)
(slot firstCharacter  'The first character in my contents' string  readWrite player getFirstCharacter  player  setFirstCharacter:)
(slot numericValue 'The number represented by my contents' number readWrite player getNumericValue player setNumericValue:)))

(basic (
(slot characters	'The characters in my contents' string	readWrite player getCharacters player setCharacters:))))


! !


!TileMorph methodsFor: 'initialization' stamp: 'sw 9/8/2000 11:10'!
setSlotRefOperator: aString
	"aString represents the name of a slot that the receiver is to represent; configure the receiver to serve thi duty, and set upthe wording on the tile appropriately"

	type _ #operator.
	operatorOrExpression _ ScriptingSystem getterSelectorFor: aString.
	operatorReadoutString _ aString.
 	self line1: aString! !


!UserScript methodsFor: 'script editor' stamp: 'sw 9/8/2000 16:38'!
instantiatedScriptEditor
	"return the current script editor, creating it if necessary"

	self isAnonymous ifTrue:
		[currentScriptEditor _ ScriptEditorMorph new playerScripted: player].

	self isTextuallyCoded ifTrue:
			["path thought not to be reached now"
			^ player costume pasteUpMorph scriptorForTextualScript: selector ofPlayer: player].

	currentScriptEditor ifNil:
		[currentScriptEditor _ (player class includesSelector: selector) 
			ifTrue: [ScriptEditorMorph new 
				fromExistingMethod: selector 
				forPlayer: player]
			ifFalse: [ScriptEditorMorph new
				setMorph: player costume
				scriptName: selector].
		status == #ticking ifTrue: [player costume arrangeToStartStepping]].
	
	^ currentScriptEditor! !

ViewerEntry removeSelector: #slotDocumentation!

!TextMorph class reorganize!
('scripting' additionsToViewerCategories authoringPrototype)
('user interface' includeInNewMorphMenu)
('class initialization' initialize)
!


!StandardViewer reorganize!
('categories' addCategoryViewer addCategoryViewerFor: categoriesCurrentlyShowing categoryMorphs likelyCategoryToShow)
('user interface' dismiss downArrowHit hasDismissButton standardViewerColor upArrowHit)
('initialization' addHeaderMorphWithBarHeight: addHeaderMorphWithBarHeight:includeDismissButton: initialHeightToAllow initializeFlapVersionFor: initializeFor:barHeight: initializeFor:barHeight:includeDismissButton: initializeFor:barHeight:includeDismissButton:showCategories:)
('object fileIn' convertbosfcebbochvimolpsf0:bosfcebbochvimlpsf0:)
!

StandardScriptingSystem initialize!
StandardScriptingSystem removeSelector: #categoryElementsFor:!
StandardScriptingSystem removeSelector: #initCategoryElementDictionary!
StandardScriptingSystem removeSelector: #initStandardScriptInfo!
StandardScriptingSystem removeSelector: #initStandardSlotInfo!
StandardScriptingSystem removeSelector: #initializeScriptHelp!
StandardScriptingSystem removeSelector: #scriptInfoFor:!
StandardScriptingSystem removeSelector: #slotInfoFor:!
Object subclass: #StandardScriptingSystem
	instanceVariableNames: ''
	classVariableNames: 'CustomPartsBin FormDictionary HelpStrings ReservedScriptNames StandardPartsBin StandardSlotInfo SystemSlotDictionary TypeColorDictionary '
	poolDictionaries: ''
	category: 'Morphic-Scripting'!

!StandardScriptingSystem reorganize!
('font & color choices' colorBehindTiles fontForNameEditingInScriptor fontForTiles statusColorSymbolFor: uniformTileInteriorColor)
('form dictionary' convertAllKeysToSymbols deletePrivateGraphics deletePrivateGraphics:afterStoringToFileNamed: formAtKey: inspectFormDictionary installSolidMenuForm mergeGraphicsFrom: patchInNewStandardPlayerForm readFormsFromFileNamed: restorePrivateGraphics saveForm:atKey: saveFormsToFileNamed: squeakyMouseForm standardFormKeys standardForms)
('help dictionary' helpStringFor: initReservedScriptNames initializeHelpStrings statusHelpString)
('utilities' cleanupsForRelease customizeForEToyUsers: getterSelectorFor: guessDOLProxy holderWithAlphabet makeAnonymousScriptsPermanent newScriptingSpace prepareForExternalReleaseNamed: reclaimSpace reinvigorateThumbnailsInViewerFlapTabs setterSelectorFor: spaceReclaimed standardEventStati stripGraphicsForExternalRelease tryButtonFor: typeForSystemSlotNamed: wordingForAssignmentSuffix: wordingForOperator:)
('parts bin' customPartsBin newStandardPartsBin prototypicalHolder resetStandardPartsBin setCustomPartsBinFrom:)
('universal slots & scripts' acceptableScriptNameFrom:forScriptCurrentlyNamed:asScriptNameIn:world: acceptableSlotNameFrom:forSlotCurrentlyNamed:asSlotNameIn:world: doesOperatorWantArrows: initializeSystemSlotDictionary isAcceptablePlayerSlotName: isSystemScriptName:)
('script-control' goButton goUp:with: scriptControlButtons stepButton stepDown:with: stepStillDown:with: stepUp:with: stopButton stopUp:with:)
('tile colors' brightColorFor: brightColorForType: colorForType: colorFudge initializeTypeColors unbrightColorFor:)
('stripped')
!

SpeakerMorph class removeSelector: #categoryContributions!
SpeakerMorph class removeSelector: #helpContributions!
SpeakerMorph class removeSelector: #standardSlotInfo!

!SpeakerMorph class reorganize!
('scripting' additionsToViewerCategories)
!

SpeakerMorph removeSelector: #addCostumeSpecificCategoriesTo:!
SimpleSliderMorph class removeSelector: #categoryContributions!
SimpleSliderMorph class removeSelector: #helpContributions!
SimpleSliderMorph class removeSelector: #standardSlotInfo!

!SimpleSliderMorph class reorganize!
('scripting' additionsToViewerCategories authoringPrototype defaultNameStemForInstances)
!

SimpleSliderMorph removeSelector: #addCostumeSpecificCategoriesTo:!
ReferenceMorph class removeSelector: #categoryContributions!

!ReferenceMorph class reorganize!
('scripting' additionsToViewerCategories defaultNameStemForInstances)
('instance creation' forMorph: forMorph:font:)
!

ReferenceMorph removeSelector: #addCostumeSpecificCategoriesTo:!
Player removeSelector: #phraseSpecFor:!
Player removeSelector: #removeScript:!
Player removeSelector: #slotPhraseFor:inViewer:!
Player removeSelector: #worldTilePhrasesSpecsForCategory:!

!Player reorganize!
('card/stack commands' createNewCard deleteCard firstPage goToNextCard goToPreviousCard goto: lastPage newCard nextPage previousPage)
('clone' copyStateFrom: copyUniClass initializeCostumesFrom: justClonedFrom:)
('costume' availableCostumeNames availableCostumesForArrows costume costume: costumeRespondingTo: costumes costumesDo: forgetOtherCostumes getValueFromCostume: hasAnyBorderedCostumes hasCostumeThatIsAWorld hasOnlySketchCostumes knownSketchCostumeWithSameFormAs: newCostume rawCostume: rememberCostume: renderedCostume: resetCostumeList sendMessageToCostume: sendMessageToCostume:with: setCostumeSlot:toValue: wearCostumeOf: wearCostumeOfClass: wearCostumeOfName: wearSketchCostumeResembling: wearSketchCostumeWithForm:using:)
('heading' headDown headLeft headRight headUp)
('name' externalName renameTo:)
('pen' addPlayerMenuItemsTo:hand: choosePenColor: choosePenSize clearTurtleTrails colorUnder defaultPenColor defaultPenSize getPenColor getPenDown getPenSize liftAllPens liftPen lowerAllPens lowerPen penColor: setPenColor:)
('slots-kernel' categories categoriesForWorld slotInfo slotNames tileForArgType: tileForArgType:inViewer: tilePhrasesSpecsForCategory: tilePhrasesSpecsInstanceVariablesCategory typeForSlot: usablePhraseSpecsIn:)
('slot getters/setters' getAmount getAngle getBorderColor getBorderWidth getBottom getCharacters getColor getColorUnder getConePosition getCursor getCursorWrapped getDescending getFirstCharacter getHeading getHeadingUnrounded getHeight getIndexInOwner getIsUnderMouse getKnobColor getLastValue getLeft getLeftRight getMaxVal getMinVal getMouseX getMouseY getName getNewClone getNumberAtCursor getNumericValue getRight getRotationStyle getSampleAtCursor getScaleFactor getSpeed getTop getTruncate getUpDown getValueAtCursor getWidth getX getY handTheUserACopy setBorderColor: setBorderWidth: setBottom: setCharacters: setColor: setConePosition: setCursor: setCursorWrapped: setDescending: setFirstCharacter: setHeading: setHeight: setIndexInOwner: setKnobColor: setLastValue: setLeft: setMaxVal: setMinVal: setName: setNumberAtCursor: setNumericValue: setRight: setRotationStyle: setSampleAtCursor: setScaleFactor: setTop: setTruncate: setValueAtCursor: setWidth: setX: setY:)
('slots-user' addInstanceVariable addInstanceVariableNamed:type:value: addSlotNamedLike:withValue: changeVariableType chooseSlotTypeFor: chooseUserSlot compileInstVarAccessorsFor: hasUserDefinedScripts hasUserDefinedSlots initialTypeForSlotNamed: initialValueForSlotOfType: removeSlotNamed: renameSlot: renameSlot:newSlotName: setPrecisionFor: slotInfoAt: tearOffFancyWatcherFor: tearOffWatcherFor: valueOfType:from:)
('slots-assignment' assignDecrGetter:setter:amt: assignGetter:setter:amt: assignIncrGetter:setter:amt: assignMultGetter:setter:amt:)
('scripts-kernel' acceptScript:for: allScriptEditors anonymousScriptEditorFor: commandPhraseFor:inViewer: elementTypeFor: expungeEmptyScripts expungeEmptyUnRenamedScripts hasScriptInvoking:ofPlayer: hasScriptReferencing:ofPlayer: infoFor:inViewer: instantiatedUserScriptsDo: isEmptyTileScript: isExpendableScript: noteRenameOf:to:inPlayer: okayToDestroyScriptNamed: okayToRemoveSlotNamed: pacifyScript: permanentScriptEditorFor: removeScript:fromWorld: removeScriptWithoutUpdatingViewers: removeScriptWithoutUpdatingViewers:fromWorld: renameScript: renameScript:newSelector: runScript: scriptEditorFor: scriptEvaluatorFor:phrase: scriptInstantiationForSelector: scriptorsForSelector:inWorld: slotInfoButtonHitFor:inViewer: sourceCodeFor: tilePhrasesForCategory:inViewer: tileScriptCommands)
('scripts-execution' assureEventHandlerRepresentsStatus runAllClosingScripts runAllOpeningScripts runAllTickingScripts scriptPerformer scriptingError: startRunning startRunningScripts step stopProgramatically stopRunning wantsSteps)
('scripts-standard' append: beep: bounce: changeScript:toStatus: clear doMenuItem: followPath forward: getObtrudes goToRightOf: hide initiatePainting loadSineWave loadSound: makeNewDrawingIn: menuItemAfter: menuItemBefore: moveToward: pauseScript: play removeAll reverse setPenDown: setPenSize: show shuffleContents startScript: stopScript: turn: wrap)
('misc' actorState allOpenViewers assureUniClass basicType beNotZero: belongsToUniClass browsePlayerClass color color:sees: defaultFloatPrecisionFor: defaultLabelForInspector dummy getIsOverColor: grabPlayerIn: isFlagshipForClass makeBounceSound: noteDeletionOf:fromWorld: presenter printOn: revealPlayerIn: revertToUnscriptedPlayerIfAppropriate roundUpStrays seesColor: stringForReadout tearOffTileForSelf thumbnailMenuEvt:forMorph: tileReferringToSelf unhideHiddenObjects unusedScriptName updateAllViewers updateAllViewersAndForceToShow: width)
('object fileIn')
('fileIn/Out' releaseCachedState storeDataOn:)
!

PasteUpMorph class removeSelector: #categoryContributions!
PasteUpMorph class removeSelector: #scriptInfo!
PasteUpMorph class removeSelector: #standardSlotInfo!

!PasteUpMorph class reorganize!
('scripting' additionsToViewerCategories authoringPrototype defaultNameStemForInstances)
('project' MinCycleLapse: disableDeferredUpdates disableDeferredUpdates: initialize newWorldForProject: newWorldTesting startUp)
!

PasteUpMorph removeSelector: #addCostumeSpecificCategoriesTo:!
PaintBoxMorph removeSelector: #addCostumeSpecificCategoriesTo:!
JoystickMorph class removeSelector: #categoryContributions!
JoystickMorph class removeSelector: #standardSlotInfo!

!JoystickMorph class reorganize!
('scripting' additionsToViewerCategories authoringPrototype)
('new morph menu' includeInNewMorphMenu)
!

GraphMorph class removeSelector: #categoryContributions!
GraphMorph class removeSelector: #helpContributions!
GraphMorph class removeSelector: #scriptInfo!
GraphMorph class removeSelector: #standardSlotInfo!

!GraphMorph class reorganize!
('scripting' additionsToViewerCategories)
!

BookMorph class removeSelector: #categoryContributions!
BookMorph class removeSelector: #scriptInfo!

!BookMorph class reorganize!
('scripting' additionsToViewerCategories authoringPrototype)
('url' alreadyInFromUrl: grabURL: isInWorld:withUrl:)
!

Morph class removeSelector: #categoryContributions!
Morph class removeSelector: #scriptInfo!
Morph class removeSelector: #standardSlotInfo!

!Morph class reorganize!
('instance creation' includeInNewMorphMenu new newBounds: newBounds:color: newSticky)
('class initialization' initialize)
('misc' morphsUnknownToTheirOwners)
('scripting' additionsToViewerCategories additionsToViewerCategory: authoringPrototype helpContributions)
!

JoystickMorph removeSelector: #addCostumeSpecificCategoriesTo:!
GraphMorph removeSelector: #addCostumeSpecificCategoriesTo:!

!DataMorph reorganize!
('data' becomeField dockUpToInstance: holdsDataForEachInstance setLiteral:)
('scripting' initializeAsAuthoringPrototypeOfType:)
('ui' wantsKeyboardFocusFor: wouldAcceptKeyboardFocus)
('display' drawOn:)
!

CategoryViewer removeSelector: #phraseForSlot:type:readOnly:getSelector:putSelector:!
BookMorph removeSelector: #addCostumeSpecificCategoriesTo:!
Morph removeSelector: #addCostumeSpecificCategoriesTo:!
"Postscript:"
StandardScriptingSystem initialize.
Preferences okToReinitializeFlaps ifTrue: [Utilities reinstateDefaultFlaps].!

Object subclass: #StandardScriptingSystem
	instanceVariableNames: ''
	classVariableNames: 'CustomPartsBin FormDictionary HelpStrings ReservedScriptNames StandardPartsBin StandardSlotInfo SystemSlotDictionary TypeColorDictionary '
	poolDictionaries: ''
	category: 'Morphic-Scripting'!

