'From Squeak 2.3 of January 14, 1999 on 8 March 1999 at 10:42:27 pm'!
"Change Set:		bookControls-sw
Date:			8 March 1999
Author:			Scott Wallace

Improves the default book controls:

(1)  Two variants available: short controls and full controls, with an easy way to go from one to the other.

(2)  The individual controls are more nicely laid out -- padding between successive items; the entire control panel centered horizontally within its container; unneeded button borders removed."!


!AlignmentMorph class methodsFor: 'instance creation' stamp: 'sw 3/2/1999 14:46'!
newVariableTransparentSpacer
	"Answer a space-filling instance of me of the given color."

	^ self new
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		inset: 0;
		borderWidth: 0;
		color: Color transparent
! !


!BookMorph class methodsFor: 'all' stamp: 'sw 3/6/1999 01:21'!
authoringPrototype
	"Answer an instance of the receiver suitable for placing in a parts bin for authors"
	
	| book |
	book _ self new markAsPartsDonor.
	book removeEverything; pageSize: 360@228; color: (Color gray: 0.9).
	book borderWidth: 1; borderColor: Color black.
	book beSticky.
	book showPageControls; insertPage.
	^ book! !


!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/2/1999 14:21'!
fewerPageControls
	self showPageControls: self shortControlSpecs! !

!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/8/1999 10:22'!
fullControlSpecs

	^ #(
		spacer
		variableSpacer
		('-'			deletePage		'Delete this page')
		spacer
		( ''		firstPage		'First page')
		spacer
		( '<' 		previousPage	'Previous page')
		spacer
		(''		invokeBookMenu 'Click here to get a menu of options for this book.')
		spacer
		('>'			nextPage		'Next page')
		spacer
		( ''		lastPage			'Final page')
		spacer
		('+'			insertPage		'Add a new page after this one')
		variableSpacer
		(''			fewerPageControls	'Fewer controls')
)! !

!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/6/1999 01:21'!
makePageControlsFrom: controlSpecs

	| c aButton col row b |
	c _ (color saturation > 0.1) ifTrue: [color slightlyLighter] ifFalse: [color slightlyDarker].
	aButton _ SimpleButtonMorph new target: self; borderWidth: 1; borderColor: Color veryLightGray; color: c.
	col _ AlignmentMorph newColumn.
	col color: c; borderWidth: 0; inset: 0.
	col hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.

	row _ AlignmentMorph newRow.
	row color: c; borderWidth: 0; inset: 0.
	row hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.
	controlSpecs do: [:spec |
		spec == #spacer
			ifTrue:
				[row addTransparentSpacerOfSize: (10 @ 0)]
			ifFalse:
				[spec == #variableSpacer
					ifTrue:
						[row addMorphBack: AlignmentMorph newVariableTransparentSpacer]
					ifFalse:
						[b _ aButton fullCopy
						label: spec first;
						actionSelector: spec second;
						borderWidth: 0;
	 					setBalloonText: spec third.
						row addMorphBack: b.
						(spec last asLowercase includesSubString: 'menu')
							ifTrue: [b actWhen: #buttonDown]]]].  "pop up menu on mouseDown"
		col addMorphBack: row.
	^ col! !

!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/8/1999 10:22'!
shortControlSpecs
	^ #(
		spacer
		variableSpacer
		( '<'	previousPage		'Previous page')
		spacer
		(''		invokeBookMenu 'Click here to get a menu of options for this book.')
		spacer
		('>'	nextPage			'Next page')
		variableSpacer
		(''	showMoreControls	'More controls'))! !

!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/2/1999 15:01'!
showPageControls
	self showPageControls: self shortControlSpecs! !

!BooklikeMorph methodsFor: 'page controls' stamp: 'sw 3/7/1999 13:07'!
showPageControls: controlSpecs
	| spacer pageControls anIndex |
	self hidePageControls.
	anIndex _ (submorphs size > 0 and: [submorphs first hasProperty: #header])
		ifTrue:	[2]
		ifFalse:	[1].
	spacer _ Morph new color: color; extent: 0@10.
	spacer  setProperty: #pageControl toValue: true.
	self privateAddMorph: spacer atIndex: anIndex.

	pageControls _ self makePageControlsFrom: controlSpecs.
	pageControls borderWidth: 0; inset: 4.
	pageControls  setProperty: #pageControl toValue: true.
	pageControls setNameTo: 'Page Controls'.
	self privateAddMorph:  pageControls beSticky atIndex: anIndex! !


!BookMorph methodsFor: 'navigation' stamp: 'sw 3/2/1999 14:23'!
showMoreControls
	self showPageControls: self fullControlSpecs! !

!BookMorph methodsFor: 'menu' stamp: 'sw 3/2/1999 14:20'!
addBookMenuItemsTo: aMenu hand: aHandMorph
	| controlsShowing subMenu |
	subMenu _ MenuMorph new defaultTarget: self.
	subMenu add: 'previous page' action: #previousPage.
	subMenu add: 'next page' action: #nextPage.
	subMenu add: 'insert a page' action: #insertPage.
	subMenu add: 'delete this page' action: #deletePage.

	controlsShowing _ self hasSubmorphWithProperty: #pageControl.
	controlsShowing
		ifTrue:
			[subMenu add: 'hide page controls' action: #hidePageControls.
			subMenu add: 'fewer page controls' action: #fewerPageControls]
		ifFalse:
			[subMenu add: 'show page controls' action: #showPageControls].

	subMenu addLine.
	subMenu add: 'sound effect for all pages' action: #menuPageSoundForAll:.
	subMenu add: 'sound effect this page only' action: #menuPageSoundForThisPage:.
	subMenu add: 'visual effect for all pages' action: #menuPageVisualForAll:.
	subMenu add: 'visual effect this page only' action: #menuPageVisualForThisPage:.

	subMenu addLine.
	subMenu add: 'sort pages' action: #sortPages:.
	subMenu add: 'uncache page sorter' action: #uncachePageSorter.
	(self hasProperty: #dontWrapAtEnd)
		ifTrue: [subMenu add: 'wrap after last page' selector: #setWrapPages: argument: true]
		ifFalse: [subMenu add: 'stop at last page' selector: #setWrapPages: argument: false].

	subMenu addLine.
	subMenu add: 'search for text' action: #textSearch.
	(aHandMorph classOfPasteBuffer isKindOf: PasteUpMorph class) ifTrue:
		[subMenu add: 'paste book page'	action: #pasteBookPage].

	subMenu add: 'send all pages to server' action: #savePagesOnURL.
	subMenu add: 'send this page to server' action: #saveOneOnURL.
	subMenu add: 'reload all from server' action: #reload.
	subMenu add: 'copy page url to clipboard' action: #copyUrl.
	subMenu add: 'keep in one file' action: #keepTogether.
	subMenu add: 'save as new-page prototype' action: #setNewPagePrototype.
	newPagePrototype ifNotNil:
		[subMenu add: 'clear new-page prototype' action: #clearNewPagePrototype].

	aMenu add: 'book...' subMenu: subMenu
! !

!BookMorph methodsFor: 'menu' stamp: 'sw 3/2/1999 14:28'!
invokeBookMenu
	"Invoke the book's control panel menu."
	| aMenu hand controlsShowing |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addList:
		#(('sort pages'				sortPages)
		('uncache page sorter'	uncachePageSorter)).
	(self hasProperty: #dontWrapAtEnd)
		ifTrue: [aMenu add: 'wrap after last page' selector: #setWrapPages: argument: true]
		ifFalse: [aMenu add: 'stop at last page' selector: #setWrapPages: argument: false].
	aMenu addList:
		#(('make bookmark'		bookmarkForThisPage)
		('make thumbnail'		thumbnailForThisPage)).
	controlsShowing _ self hasSubmorphWithProperty: #pageControl.
	controlsShowing
		ifTrue:
			[aMenu add: 'hide page controls' action: #hidePageControls.
			aMenu add: 'fewer page controls' action: #fewerPageControls]
		ifFalse:
			[aMenu add: 'show page controls' action: #showPageControls].

	aMenu addLine.
	aMenu add: 'sound effect for all pages' action: #menuPageSoundForAll:.
	aMenu add: 'sound effect this page only' action: #menuPageSoundForThisPage:.
	aMenu add: 'visual effect for all pages' action: #menuPageVisualForAll:.
	aMenu add: 'visual effect this page only' action: #menuPageVisualForThisPage:.

	aMenu addLine.
	(self primaryHand classOfPasteBuffer isKindOf: PasteUpMorph class) ifTrue:
		[aMenu add: 'paste book page'	action: #pasteBookPage].

	aMenu add: 'save as new-page prototype' action: #setNewPagePrototype.
	newPagePrototype ifNotNil: [
		aMenu add: 'clear new-page prototype' action: #clearNewPagePrototype].

	aMenu add: (openToDragNDrop ifTrue: ['close'] ifFalse: ['open']) , ' dragNdrop'
			action: #openCloseDragNDrop.
	aMenu add: 'make all pages this size' action: #makeUniformPageSize.
	aMenu add: 'search for text' action: #textSearch.
	aMenu add: 'send all pages to server' action: #savePagesOnURL.
	aMenu add: 'send this page to server' action: #saveOneOnURL.
	aMenu add: 'reload all from server' action: #reload.
	aMenu add: 'copy page url to clipboard' action: #copyUrl.
	aMenu add: 'keep in one file' action: #keepTogether.

	hand _ self world primaryHand.
	aMenu popUpAt: hand position forHand: hand.
"
	sel _ aMenu invokeAt: self primaryHand position in: self world.
	sel ifNotNil: [self perform: sel].
"! !


!Color methodsFor: 'transformations' stamp: 'sw 3/6/1999 01:19'!
slightlyDarker
	"Answer a slightly darker shade of this color."

	^ self mixed: 0.93 with: Color black
! !

!Color methodsFor: 'transformations' stamp: 'sw 3/6/1999 01:20'!
slightlyLighter
	"Answer a slightly lighter shade of this color."

	^ self mixed: 0.93 with: Color white
! !


"Postscript:
"
Utilities reinstateDefaultFlaps.
ScriptingSystem resetStandardPartsBin.
!

