'From Squeak3.7-m17n of 30 June 2004 [latest update: #16] on 21 July 2004 at 12:46:33 pm'!
Object subclass: #LanguageEnvironment
	instanceVariableNames: 'id translator '
	classVariableNames: 'ClipboardInterpreterClass FileNameConverterClass InputInterpreterClass SystemConverterClass Current KnownEnvironments '
	poolDictionaries: ''
	category: 'Multilingual-Languages'!

!Clipboard methodsFor: 'accessing' stamp: 'mir 7/20/2004 15:44'!
setInterpreter

	interpreter _ LanguageEnvironment defaultClipboardInterpreter.
	interpreter ifNil: [
		"Should never be reached, but just in case."
		interpreter _ NoConversionClipboardInterpreter new].
! !


!FilePath methodsFor: 'conversion' stamp: 'mir 7/20/2004 15:48'!
pathName: p isEncoded: isEncoded

	converter _ LanguageEnvironment defaultFileNameConverter.
	isEncoded ifTrue: [
		squeakPathName _ p convertFromWithConverter: converter.
		systemPathName _ p.
	] ifFalse: [
		squeakPathName _ p isOctetString ifTrue: [p asOctetString] ifFalse: [p].
		systemPathName _ squeakPathName convertToWithConverter: converter.
	].
! !


!HandMorph methodsFor: 'multilingual' stamp: 'mir 7/15/2004 18:53'!
keyboardInterpreter

	^keyboardInterpreter ifNil: [keyboardInterpreter _ LanguageEnvironment current class defaultInputInterpreter]! !


!LanguageEnvironment methodsFor: 'initialize-release' stamp: 'mir 7/15/2004 15:46'!
beCurrentNaturalLanguage

! !

!LanguageEnvironment methodsFor: 'initialize-release' stamp: 'mir 7/15/2004 15:31'!
localeID: anID
	id := anID! !

!LanguageEnvironment methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:32'!
isoCountry
	^self localeID isoCountry! !

!LanguageEnvironment methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:32'!
isoLanguage
	^self localeID isoLanguage! !

!LanguageEnvironment methodsFor: 'accessing' stamp: 'mir 7/15/2004 18:55'!
leadingChar
	^self class leadingChar! !

!LanguageEnvironment methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:31'!
localeID
	^id! !


!JapaneseEnvironment methodsFor: 'initialize-release' stamp: 'mir 7/15/2004 15:46'!
beCurrentNaturalLanguage

	super beCurrentNaturalLanguage.
	Preferences restoreDefaultFontsForJapanese.
! !


!KoreanEnvironment methodsFor: 'initialize-release' stamp: 'mir 7/15/2004 15:46'!
beCurrentNaturalLanguage

	super beCurrentNaturalLanguage.
	Preferences restoreDefaultFontsForJapanese.
! !


!LanguageEnvironment class methodsFor: 'class initialization' stamp: 'mir 7/15/2004 15:54'!
initialize
	"LanguageEnvironment initialize"

	Smalltalk addToStartUpList: LanguageEnvironment after: FileDirectory.
	Smalltalk addToStartUpList: FileDirectory after: LanguageEnvironment.
! !

!LanguageEnvironment class methodsFor: 'class initialization' stamp: 'mir 7/15/2004 16:13'!
localeChanged
	self startUp! !

!LanguageEnvironment class methodsFor: 'class initialization' stamp: 'mir 7/15/2004 15:54'!
startUp

	self clearDefault.
	Current := nil.
	Clipboard startUp.
	HandMorph startUp.
! !

!LanguageEnvironment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/1/2004 17:59'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	self subclassResponsibility! !

!LanguageEnvironment class methodsFor: 'public query' stamp: 'mir 7/21/2004 11:26'!
defaultClipboardInterpreter

	ClipboardInterpreterClass ifNil: [ClipboardInterpreterClass _ self current class clipboardInterpreterClass].
	^ ClipboardInterpreterClass new.

! !

!LanguageEnvironment class methodsFor: 'public query'!
defaultFileNameConverter
	FileNameConverterClass
		ifNil: [FileNameConverterClass := self current class fileNameConverterClass].
	^ FileNameConverterClass new! !

!LanguageEnvironment class methodsFor: 'public query' stamp: 'mir 7/21/2004 11:31'!
defaultInputInterpreter

	InputInterpreterClass ifNil: [InputInterpreterClass _ self current class inputInterpreterClass].
	^ InputInterpreterClass new.
! !

!LanguageEnvironment class methodsFor: 'public query' stamp: 'mir 7/21/2004 11:31'!
defaultSystemConverter

	SystemConverterClass ifNil: [SystemConverterClass _ self current class systemConverterClass].
	^ SystemConverterClass new.
! !

!LanguageEnvironment class methodsFor: 'accessing' stamp: 'mir 7/15/2004 18:39'!
current
	"LanguageEnvironment current"
	^Current ifNil: [
		Current := Locale current languageEnvironment.
		Current beCurrentNaturalLanguage.
		^Current]! !

!LanguageEnvironment class methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:45'!
localeID: localeID
	^self knownEnvironments at: localeID ifAbsentPut: [self new localeID]! !

!LanguageEnvironment class methodsFor: 'private' stamp: 'mir 7/15/2004 15:44'!
initKnownEnvironments
	"LanguageEnvironment initKnownEnvironments"

	| envMap env known |
	envMap := Dictionary new.
	known := Dictionary new.
	self allSubclassesDo: [:subClass | 
		subClass supportedLanguages do: [:language | 
			envMap at: language put: subClass]].
	{ {'en'. nil.}. {'de'. nil.}. {'sv'. nil.}. {'nl'. nil.}. {'no'. nil.}. {'sv'. nil.}. {'ja'.nil.}. {'ja'.'etoy'.} }
		do: [:spec | 
			env := (envMap at: spec first ifAbsent: [LanguageEnvironment]) new.
			env localeID: (LocaleID isoLanguage: spec first isoCountry: spec second).
			known at: env localeID put: env].
	^known! !

!LanguageEnvironment class methodsFor: 'private' stamp: 'mir 7/15/2004 15:45'!
knownEnvironments
	"LanguageEnvironment knownEnvironments"
	"KnownEnvironments := nil"

	^KnownEnvironments ifNil: [KnownEnvironments := self initKnownEnvironments]! !


!GreekEnvironment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/1/2004 18:23'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	
	^#('el' )! !


!JapaneseEnvironment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/1/2004 18:03'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	
	^#('ja' )! !


!KoreanEnvironment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/1/2004 18:42'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	^#('ko' )! !


!Latin1Environment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/15/2004 19:19'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	
	^#('en' 'de' 'fr' 'no' 'sv' 'nl')! !


!MultiString methodsFor: 'converting' stamp: 'mir 7/20/2004 15:49'!
convertToSystemString

	| readStream writeStream converter |
	readStream _ self readStream.
	writeStream _ String new writeStream.
	converter _ LanguageEnvironment defaultSystemConverter.
	converter ifNil: [^ self].
	[readStream atEnd] whileFalse: [
		converter nextPut: readStream next toStream: writeStream
	].
	converter emitSequenceToResetStateIfNeededOn: writeStream.
	^ writeStream contents.
! !


!SimplifiedChineseEnvironment methodsFor: 'initialize-release' stamp: 'mir 7/15/2004 15:47'!
beCurrentNaturalLanguage

	super beCurrentNaturalLanguage.
	Preferences restoreDefaultFontsForJapanese.
! !


!SimplifiedChineseEnvironment class methodsFor: 'subclass responsibilites' stamp: 'mir 7/1/2004 18:22'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	
	^#('zh' )! !


!SpanishEnvironment class methodsFor: 'subclass responsibilities' stamp: 'mir 7/1/2004 18:02'!
supportedLanguages
	"Return the languages that this class supports. 
	Any translations for those languages will use this class as their environment."
	
	^#('es' )! !


!String methodsFor: 'converting' stamp: 'mir 7/20/2004 15:50'!
convertFromSystemString

	| readStream writeStream converter |
	readStream _ self readStream.
	writeStream _ String new writeStream.
	converter _ LanguageEnvironment defaultSystemConverter.
	converter ifNil: [^ self].
	[readStream atEnd] whileFalse: [
		writeStream nextPut: (converter nextFromStream: readStream)].
	^ writeStream contents
! !


!TextConverter class methodsFor: 'as yet unclassified' stamp: 'mir 7/20/2004 15:51'!
defaultSystemConverter

	^LanguageEnvironment defaultSystemConverter! !


!X11Encoding class methodsFor: 'as yet unclassified' stamp: 'mir 7/15/2004 18:57'!
useEncoding: encoding

	self setEncoding: encoding.
	LanguageEnvironment startUp.
	^ self encoding.! !

!X11Encoding class methodsFor: 'as yet unclassified' stamp: 'mir 7/15/2004 18:57'!
useLocaleEncoding

	self setEncodingToLocale.
	LanguageEnvironment startUp.
	^ self encoding.! !

LanguageEnvironment initialize!
Object subclass: #LanguageEnvironment
	instanceVariableNames: 'id translator'
	classVariableNames: 'ClipboardInterpreterClass Current FileNameConverterClass InputInterpreterClass KnownEnvironments SystemConverterClass'
	poolDictionaries: ''
	category: 'Multilingual-Languages'!

!FilePath reorganize!
('conversion' asSqueakPathName asSystemPathName coverter: pathName pathName:isEncoded: printOn:)
!

